import React, { useState } from 'react';
import { useSelector } from 'react-redux';
import { Row, Col } from 'reactstrap';
import { MySelect } from '../../helpers/helpersIndex';
import { EOprationalActions } from '../../models/utilitiesModel';
import DigitalFormCreateOrEditManager from './digitalFormCreateOrEditManager';

const initialState = {
    actionType: EOprationalActions.UNSELECT,
    actionData: null,
    error: ''
};
function DynamicFormBuilderParent() {
    const formOptions = useSelector((state: any) => state.dynamicFormBuilderReducer?.formBuilderData || []);
    const [state, setState] = useState(() => initialState);
    const clearSelection = () => setState(initialState);

    const selectFormType = e => setState({ ...state, actionData: e || '', error: '' });
    const addForm = () => setState({ ...state, actionType: EOprationalActions.ADD, actionData: null, error: '' });
    const editForm = () => {
        if ((state.actionData as any)?.formBuilderTitle)
            setState({ ...state, error: '', actionType: EOprationalActions.EDIT });
        else
            setState({ ...state, error: 'Select a form' });
    }
    console.log('DynamicFormBuilderParent==>', { formOptions })
    return (
        <div>
            <Row className='d-flex'>
                <Col sm="3">
                    <MySelect
                        allOption={{
                            label: "Select All",
                            value: 0
                        }}
                        value={state.actionData}
                        isMulti={false}
                        name='formBuilderLabel'
                        placeholder={'select Form'}
                        options={formOptions}
                        onChange={selectFormType}
                        getOptionLabel={option => option.formBuilderTitle}
                        getOptionValue={option => option.formBuilderTitle}
                        valueKey="value"
                        hideSelectedOptions={false}
                        removeSelected={false}
                        closeMenuOnSelect={true}
                        backspaceRemovesValue={false}
                        isSearchable={true}
                        allowSelectAll={false}
                        onBlur={() => { }}
                        noOptionsMessage={() => 'No Data Found'}
                        isDisabled={state.actionType !== EOprationalActions.UNSELECT}
                    />
                    {state.error && <span className="text-danger">{state.error}</span>}
                </Col>
                <Col sm="3" className="mt-2">
                    {state.actionType === EOprationalActions.UNSELECT && <>
                        <button onClick={addForm} type="button" className="btn btn-sm btn-outline-success mx-1">Add</button>
                        <button onClick={editForm} type="button" className="btn btn-sm btn-outline-primary mx-1">Edit</button>
                    </>}
                    {state.actionType !== EOprationalActions.UNSELECT && <button onClick={clearSelection} type="button" className="btn btn-sm btn-outline-danger mx-1">Clear</button>}
                </Col>
            </Row>
            <br />

            {state.actionType !== EOprationalActions.UNSELECT && <DigitalFormCreateOrEditManager formBuilderData={state.actionData} callbackFn={clearSelection} />}
        </div>

    )
}
export default React.memo(DynamicFormBuilderParent);