import React from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { useFormikContext, Field, Form, Formik, ErrorMessage } from 'formik';
import { FormGroup, Label, Row, Col } from 'reactstrap';
import { IFormBuilderDateControlRestrictOptions, IFormBuilderDateFormats, IFormBuilderReducer } from '../digitalFormModel';
import * as Yup from 'yup';
import { useTranslation } from 'react-i18next';
import { customContentValidation, defultContentValidate, MySelect } from '../../../helpers/helpersIndex';
import * as _ from 'lodash';
import DatePicker from "react-datepicker";
import "react-datepicker/dist/react-datepicker.css";
import { EOprationalActions, EConfirmationActionType, IConfirmOptions, IConfirmModel } from 'models/utilitiesModel';
import { addOrEditOrRemoveDigitalFormControlProperties, setConfirmationOpen } from '../../../store/actions';

const dateFormatOptions = [
    { label: 'DD-MM-YYYY', value: IFormBuilderDateFormats.DD_MM_YYYY },
    { label: 'YYYY-MM-DD', value: IFormBuilderDateFormats.YYYY_MM_DD }
];
const restrictByOptions = [
    { label: 'Select an option', value: '' },
    { label: 'Date wise restrictions', value: 'dateRange' },
    { label: 'Days', value: 'noOfDays' }
];

const DatePropertiesComp: React.FC<any> = ({ arrayHelperMethods }) => {
    const { t } = useTranslation('translations');
    const dispatch = useDispatch();

    const actionData: any = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.actionData);
    const digitalFormCreationData: any = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.digitalFormCreationData);
    const stepperFormIndex: any = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.stepperFormIndex);

    const { formControlProperties, formControlLabel, formControlType, formControlId } = actionData;
    let { digitalFormControls, digitalStepperForms, digitalFormConfigurations } = digitalFormCreationData;
    let formControlsList = digitalFormConfigurations?.isStepperForm ? digitalStepperForms?.[stepperFormIndex]?.stepperFormControls : digitalFormControls;
    let index = formControlsList?.findIndex(x => x.formControlId === formControlId);
    let mappingFormBuilderData = formControlsList?.slice(0, index)?.filter(x => x.formControlProperties?.isDependencyQuestionRequired);
    if (digitalFormConfigurations?.isStepperForm) {
        let stepperDependencyControlsList = digitalStepperForms?.slice(0, stepperFormIndex)?.flatMap(y => y.stepperFormControls)?.filter(x => x.formControlProperties?.isDependencyQuestionRequired);
        mappingFormBuilderData = [...stepperDependencyControlsList, ...mappingFormBuilderData]
    }

    const dateInitailValues = () => ({
        formControlLabel: formControlLabel ? formControlLabel : '',
        isRequired: formControlProperties ? formControlProperties.isRequired : false,
        requiredMessage: formControlProperties ? formControlProperties.requiredMessage : '',
        dateFormat: formControlProperties ? dateFormatOptions?.find(x => x.value === formControlProperties.dateFormat) : '',
        restrictBy: formControlProperties ? restrictByOptions.find(x => x.value === formControlProperties.restrictBy?.value) : '',
        noOfDays: formControlProperties ? formControlProperties.noOfDays : '',
        dateRange: formControlProperties ? formControlProperties.dateRange : '',
        fromDate: (formControlProperties && formControlProperties.fromDate) ? new Date(formControlProperties.fromDate) : '',
        toDate: (formControlProperties && formControlProperties.toDate) ? new Date(formControlProperties.toDate) : '',
        isCurrentFromDate: formControlProperties ? formControlProperties.isCurrentFromDate : '',
        isCurrentToDate: formControlProperties ? formControlProperties.isCurrentToDate : '',
        beforeOrAfter: formControlProperties ? formControlProperties.beforeOrAfter : ''
    });
    console.log('__dateInitailValues=>', { formControlProperties, dateInitailValues: dateInitailValues() });

    const dateValidationSchema = Yup.object().shape({
        formControlLabel: customContentValidation(t, t('controleErrors.required'), { patternType: 'alphanumericspacesp', message: 'alphanumaricandspace', spacialChar: '' }, 255, 1),
        requiredMessage: Yup.string().when('isRequired', {
            is: (isRequired) => isRequired,
            then: customContentValidation(t, t('controleErrors.required'), { patternType: 'alphanumericspacesp', message: 'alphanumaricandspace', spacialChar: '' }, 255, 1),
            otherWise: defultContentValidate(t('controleErrors.required'))
        }),
        dateFormat: customContentValidation(t, t('controleErrors.required')),
        noOfDays: Yup.lazy(noOfDaysVal => {
            if (noOfDaysVal)
                return customContentValidation(t, t('controleErrors.required'), { patternType: 'number', message: 'number', spacialChar: '' }, 365, 1);
            else
                return defultContentValidate('');
        }),
        // noOfDays: Yup.string().when('restrictBy', {
        //     is: restrictByVal => restrictByVal === IFormBuilderDateControlRestrictOptions.NO_OF_DAYS,
        //     then: customContentValidation(t, t('controleErrors.required'), { patternType: 'numberWithOutZero', message: 'number', spacialChar: '' }, 365, 1),
        //     otherwise: defultContentValidate('')
        // }),
        // dateRange: Yup.string().when('restrictBy', {
        //     is: restrictByVal => restrictByVal === IFormBuilderDateControlRestrictOptions.DATE_RANGE,
        //     then: customContentValidation(t, t('controleErrors.required')),
        //     otherwise: defultContentValidate('')
        // }),
    });

    const selectRequiredCheckBoxValue = (e, setFieldValue) => {
        setFieldValue('isRequired', e.target.checked);
        setFieldValue('requiredMessage', '');
    }



    const handleDateCloseChanges = () => {
        if (index !== -1 && !formControlProperties) {
            if (digitalFormConfigurations?.isStepperForm) {
                let stepperForm = digitalStepperForms[stepperFormIndex]
                stepperForm.stepperFormControls?.splice(index, 1);
                digitalStepperForms?.splice(stepperFormIndex, 1, stepperForm)
            } else {
                formControlsList?.splice(index, 1);
            }
            let formData = {
                ...digitalFormCreationData,
                ...(digitalFormConfigurations?.isStepperForm ?
                    { digitalStepperForms: [...digitalStepperForms] } :
                    { digitalFormControls: [...formControlsList] }),
            }
            dispatch(addOrEditOrRemoveDigitalFormControlProperties(formData, EOprationalActions.EDIT))
        } else {
            dispatch(addOrEditOrRemoveDigitalFormControlProperties(digitalFormCreationData, EOprationalActions.EDIT))
        }
    }

    const openConfirmationAlert = () => {
        let optionsData = [
            {
                title: 'Yes',
                loading: 0,
                className: 'btn-info',
                function: handleDateCloseChanges,
                actionType: EConfirmationActionType.COMPONENT
            },
            {
                title: 'No',
                loading: 0,
                className: 'btn-danger'
            }
        ] as IConfirmOptions[];
        let confirmModel = {
            title: 'Do You Want to Discard the Changes ?',
            subtitle: ``,
            options: optionsData,
            transKey: '1234'
        } as IConfirmModel;
        dispatch(setConfirmationOpen(confirmModel));
    }

    return (
        <Formik
            enableReinitialize
            initialValues={dateInitailValues()}
            validationSchema={dateValidationSchema}
            onSubmit={values => {
                const formControlPropertiesData = { ...values, dateFormat: (values.dateFormat as any)?.value };
                console.log('__dateControlValues', formControlProperties);
                const { formControlLabel, ...properties } = formControlPropertiesData
                if (index !== -1) {
                    let formControl = formControlsList[index]
                    formControl.formControlProperties = properties;
                    formControl.formControlLabel = formControlLabel;
                    if (digitalFormConfigurations?.isStepperForm) {
                        formControl.stepperFormId = digitalStepperForms[stepperFormIndex]?.stepperFormId
                    }
                    formControlsList?.splice(index, 1, formControl);
                }
                if (digitalFormConfigurations?.isStepperForm) {
                    let stepperForm = digitalStepperForms[stepperFormIndex]
                    stepperForm.stepperFormControls = [...formControlsList];
                    digitalStepperForms?.splice(stepperFormIndex, 1, stepperForm)
                }
                let formData = {
                    ...digitalFormCreationData,
                    ...(digitalFormConfigurations?.isStepperForm ?
                        { digitalStepperForms: [...digitalStepperForms] } :
                        { digitalFormControls: [...formControlsList] }),
                }
                dispatch(addOrEditOrRemoveDigitalFormControlProperties(formData, EOprationalActions.EDIT))

            }}
        >
            {
                ({ values, setFieldValue, errors, touched, setFieldTouched, submitForm }) => {
                    return <Form className='form-elements'>
                        <div>
                            <h6>{`${formControlType} Properties`}</h6>
                            <Row>
                                <Col sm="12">
                                    <FormGroup>
                                        <Field id="formControlLabel" type="text" placeholder="Enter Label" name="formControlLabel" className="form-control" />
                                        <ErrorMessage name="formControlLabel" component="div" className="text-danger" />
                                    </FormGroup>
                                </Col>
                            </Row>
                            <hr />
                        </div>
                        <div className="form-inputs">
                            <Row>
                                <Col sm='12'>
                                    <FormGroup>
                                        <Label htmlFor='tag'>Tag Name </Label>
                                        <Field id='tag' type='text' name='tag' placeholder='Enter Tag Name' className='form-control' />
                                        <ErrorMessage name='tag' component='div' className='text-danger' />
                                    </FormGroup>
                                </Col>
                                <Col sm="12" className="d-flex my-2">
                                    <FormGroup className='custom-control custom-checkbox mb-0 pl-0'>
                                        <Field id="isRequired" type="checkbox" name="isRequired" className="custom-control-input" onChange={(e) => selectRequiredCheckBoxValue(e, setFieldValue)} />
                                        <label className="custom-control-label" htmlFor="isRequired">Is Required :</label>
                                    </FormGroup>
                                </Col>
                                {values.isRequired && <Col sm='12'>
                                    <FormGroup>
                                        <Label htmlFor='requiredMessage'>Required Message </Label>
                                        <Field id='requiredMessage' type='text' name='requiredMessage' placeholder='Enter your Required Message' className='form-control' />
                                        <ErrorMessage name='requiredMessage' component='div' className='text-danger' />
                                    </FormGroup>
                                </Col>}
                                <Col sm='12'>
                                    <FormGroup>
                                        <Label>Date Format</Label>
                                        <MySelect
                                            allOption={{
                                                label: "Select All",
                                                value: 0
                                            }}
                                            isMulti={false}
                                            name="dateFormat"
                                            placeholder="Select a date format"
                                            options={dateFormatOptions}
                                            value={values.dateFormat}
                                            onChange={(e) => setFieldValue('dateFormat', e || '')}
                                            getOptionLabel={option => option.label}
                                            getOptionValue={option => option.value}
                                            valueKey="value"
                                            hideSelectedOptions={false}
                                            removeSelected={false}
                                            closeMenuOnSelect={true}
                                            backspaceRemovesValue={false}
                                            isSearchable={false}
                                            allowSelectAll={false}
                                            onBlur={() => setFieldTouched('dateFormat', true)}
                                            noOptionsMessage={() => 'No Data Found'}
                                            className="select-box"
                                        />
                                    </FormGroup>
                                </Col>
                                {values.dateFormat && <Col sm='12'>
                                    <FormGroup>
                                        <Label>Restrict By</Label>
                                        <MySelect
                                            allOption={{
                                                label: "Select All",
                                                value: 0
                                            }}
                                            isMulti={false}
                                            name="restrictBy"
                                            placeholder="Select restrictBy type"
                                            options={restrictByOptions}
                                            value={values.restrictBy}
                                            onChange={(e) => {
                                                setFieldValue('fromDate', '');
                                                setFieldValue('toDate', '');
                                                setFieldValue('isCurrentFromDate', '');
                                                setFieldValue('isCurrentToDate', '');
                                                setFieldValue('noOfDays', '');

                                                setFieldValue('restrictBy', e || '');
                                            }}
                                            getOptionLabel={option => option.label}
                                            getOptionValue={option => option.value}
                                            valueKey="value"
                                            hideSelectedOptions={false}
                                            removeSelected={false}
                                            closeMenuOnSelect={true}
                                            backspaceRemovesValue={false}
                                            isSearchable={false}
                                            allowSelectAll={false}
                                            onBlur={() => setFieldTouched('restrictBy', true)}
                                            noOptionsMessage={() => 'No Data Found'}
                                        />
                                        {/* <div>
                                            <Field type='radio' name='restrictBy' className="mx-1" value={IFormBuilderDateControlRestrictOptions.DATE_RANGE} />
                                            <Label>Date wise restrictions</Label>

                                            <Field type='radio' name='restrictBy' className="mx-1" value={IFormBuilderDateControlRestrictOptions.NO_OF_DAYS} />
                                            <Label>Days</Label>

                                            <ErrorMessage name='restrictBy' component='div' className='text-danger' />
                                        </div> */}
                                    </FormGroup>
                                </Col>}
                                {(values.restrictBy as any)?.value === IFormBuilderDateControlRestrictOptions.NO_OF_DAYS && <Col sm='12'>
                                    <FormGroup>
                                        <Field type="number" name="noOfDays" placeholder="Enter no.of days" className={'form-control ' + ((errors.noOfDays && touched.noOfDays) ? 'is-invalid' : '')} />
                                        <ErrorMessage name='noOfDays' component='div' className='text-danger' />
                                    </FormGroup>
                                </Col>}
                                {values.noOfDays && <Col sm='12'>
                                    <FormGroup>
                                        <Field checked={values.beforeOrAfter === "Before"} className="d-inline-block mt-2" type='radio' name='beforeOrAfter' onChange={(e) => {
                                            const isChecked = e.target.checked;
                                            setFieldValue('beforeOrAfter', isChecked ? "Before" : "After");

                                            if (isChecked) {
                                                const date = new Date();
                                                date.setDate(date.getDate() - Number(values.noOfDays));

                                                setFieldValue('fromDate', date);
                                                setFieldValue('toDate', new Date());
                                            }
                                        }} /><span className='mx-1'>Before</span>

                                        <Field checked={values.beforeOrAfter === "After"} className="d-inline-block mt-2" type='radio' name='beforeOrAfter' onChange={(e) => {
                                            const isChecked = e.target.checked;
                                            setFieldValue('beforeOrAfter', isChecked ? "After" : "Before");

                                            if (isChecked) {
                                                const date = new Date();
                                                date.setDate(date.getDate() + Number(values.noOfDays));

                                                setFieldValue('fromDate', new Date());
                                                setFieldValue('toDate', date);
                                            }
                                        }} /><span className='mx-1'>After</span>
                                    </FormGroup>
                                </Col>}
                                {(values.dateFormat && (values.restrictBy as any)?.value === IFormBuilderDateControlRestrictOptions.DATE_RANGE) &&
                                    <>
                                        <Col sm='12'>
                                            <FormGroup>
                                                <Label className="d-block">From Date</Label>
                                                {!values.isCurrentFromDate && <>
                                                    <DatePicker className="w100 datepickerIcon form-control" placeholderText="Select From Date" name="fromDate"
                                                        popperPlacement="bottom"
                                                        popperModifiers={{
                                                            flip: {
                                                                behavior: ["bottom"]
                                                            },
                                                            preventOverflow: {
                                                                enabled: false
                                                            }
                                                        }}
                                                        selected={values.fromDate || ''}
                                                        onChange={(e) => setFieldValue('fromDate', e || '')}
                                                        dateFormat={(values.dateFormat as any)?.value || ''}
                                                        // maxDate={new Date(currentDateAndTime?.date)}
                                                        onBlur={() => setFieldTouched('fromDate', true)}
                                                        autoComplete="off"
                                                    />
                                                    {errors.fromDate && touched.fromDate && (
                                                        <div className="text-danger">{errors.fromDate}</div>
                                                    )}
                                                </>}

                                                <Field className="d-inline-block mt-2" type='checkbox' name='isCurrentFromDate' onChange={(e) => {
                                                    const isChecked = e.target.checked;
                                                    setFieldValue('isCurrentFromDate', isChecked);

                                                    if (isChecked)
                                                        setFieldValue('fromDate', new Date());
                                                    else
                                                        setFieldValue('fromDate', '');
                                                }} /><span className='mx-1'> Is current date </span>
                                            </FormGroup>
                                        </Col>
                                        <Col sm='12'>
                                            <FormGroup>
                                                <Label className="d-block">To Date</Label>
                                                {!values.isCurrentToDate && <>
                                                    <DatePicker className="w100 datepickerIcon form-control" placeholderText="Select To Date" name="toDate"
                                                        popperPlacement="bottom"
                                                        popperModifiers={{
                                                            flip: {
                                                                behavior: ["bottom"]
                                                            },
                                                            preventOverflow: {
                                                                enabled: false
                                                            }
                                                        }}
                                                        selected={values.toDate || ''}
                                                        onChange={(e) => setFieldValue('toDate', e || '')}
                                                        dateFormat={(values.dateFormat as any)?.value || ''}
                                                        // maxDate={new Date(currentDateAndTime?.date)}
                                                        onBlur={() => setFieldTouched('toDate', true)}
                                                        autoComplete="off"
                                                    />
                                                    {errors.toDate && touched.toDate && (
                                                        <div className="text-danger">{errors.toDate}</div>
                                                    )}
                                                </>}

                                                <Field className="d-inline-block mt-2" type='checkbox' name='isCurrentToDate' onChange={(e) => {
                                                    const isChecked = e.target.checked;
                                                    setFieldValue('isCurrentToDate', isChecked);

                                                    if (isChecked)
                                                        setFieldValue('toDate', new Date());
                                                    else
                                                        setFieldValue('toDate', '');
                                                }} /><span className='mx-1'> Is current date </span>
                                            </FormGroup>
                                        </Col>
                                    </>}
                            </Row>
                        </div>
                        <div className='py-2'>
                            <button className='button button-clear mr-2' type='button' onClick={() => openConfirmationAlert()}>Close</button>
                            <button className='button button-save-bordered' type='submit'>Save</button>
                        </div>
                    </Form>
                }
            }
        </Formik>
    )
}

export default React.memo(DatePropertiesComp)