import React from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { ModalFooter } from 'reactstrap';
import { useFormikContext, Field, Form, Formik, FieldArray, ErrorMessage } from 'formik';
import { FormGroup, Label, Row, Col } from 'reactstrap';
import { EDisplayType, IFormBuilderReducer } from '../digitalFormModel';
import * as Yup from 'yup';
import { useTranslation } from 'react-i18next';
import { customContentValidation, defultContentValidate, MySelect } from '../../../helpers/helpersIndex';
import * as _ from 'lodash';
import { alertActionRequest, addOrEditOrRemoveDigitalFormControlProperties, setConfirmationOpen } from '../../../store/actions';
import { EOprationalActions, EConfirmationActionType, IConfirmOptions, IConfirmModel } from '../../../models/utilitiesModel';

const RadioButtonPropertiesComp: React.FC<any> = ({ arrayHelperMethods }) => {

    const { t } = useTranslation('translations');
    const dispatch = useDispatch();

    const actionData: any = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.actionData);
    const digitalFormCreationData: any = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.digitalFormCreationData);

    const stepperFormIndex: any = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.stepperFormIndex);

    const { formControlProperties, formControlLabel, formControlType, formControlId } = actionData;
    let { digitalFormControls, digitalStepperForms, digitalFormConfigurations } = digitalFormCreationData;
    let formControlsList = digitalFormConfigurations?.isStepperForm ? digitalStepperForms?.[stepperFormIndex]?.stepperFormControls : digitalFormControls;
    let index = formControlsList?.findIndex(x => x.formControlId === formControlId);
    let mappingFormBuilderData = formControlsList?.slice(0, index)?.filter(x => x.formControlProperties?.isDependencyQuestionRequired);
    if (digitalFormConfigurations?.isStepperForm) {
        let stepperDependencyControlsList = digitalStepperForms?.slice(0, stepperFormIndex)?.flatMap(y => y.stepperFormControls)?.filter(x => x.formControlProperties?.isDependencyQuestionRequired);
        mappingFormBuilderData = [...stepperDependencyControlsList, ...mappingFormBuilderData]
    }

    const optionsData = {
        label: '',
        value: '',
        isdefault: false
    };

    const radioButtonValidationSchema = Yup.object().shape({
        formControlLabel: customContentValidation(t, t('controleErrors.required'), { patternType: 'alphanumericspacesp', message: 'alphanumaricandspace', spacialChar: '' } as any, 255, 1),
        requiredMessage: Yup.string().when('isRequired', {
            is: (isRequired) => isRequired,
            then: customContentValidation(t, t('controleErrors.required'), { patternType: 'alphanumericspacesp', message: 'alphanumaricandspace', spacialChar: '' } as any, 255, 1),
            otherWise: defultContentValidate(t('controleErrors.required'))
        }),
        optionsData: Yup.array().of(
            Yup.object().shape({
                label: customContentValidation(t, t('controleErrors.required'), { patternType: 'alphanumericspacesp', message: 'alphanumaricandspace', spacialChar: '' } as any, 255, 1),
                value: customContentValidation(t, t('controleErrors.required'), { patternType: 'alphanumericspacesp', message: 'alphanumaricandspace', spacialChar: '' } as any, 255, 1),
            })
        ),
        dependencyQuestionType: Yup.string().when('isDependencyQuestionRequired', {
            is: (isDependencyQuestionRequired) => isDependencyQuestionRequired,
            then: defultContentValidate(t('controleErrors.required')),
            otherWise: defultContentValidate('')
        }),
        dependencyCreation: Yup.string().when('dependencyQuestionType', {
            is: (dependencyQuestionType) => dependencyQuestionType === 'custome',
            then: defultContentValidate(t('controleErrors.required')),
            otherWise: defultContentValidate('')
        }),
    })
    const radioButtonsInitailValues = () => ({
        formControlLabel: formControlLabel ? formControlLabel : "",
        tag: formControlProperties ? formControlProperties.tag : "",
        isRequired: formControlProperties ? formControlProperties.isRequired : false,
        requiredMessage: formControlProperties ? formControlProperties.requiredMessage : '',
        displayType: formControlProperties ? formControlProperties.displayType : EDisplayType.Horizontal,
        optionsData: formControlProperties ? formControlProperties.optionsData : [optionsData],
        isDependencyQuestionRequired: formControlProperties ? formControlProperties.isDependencyQuestionRequired : false,
        dependencyQuestionType: formControlProperties ? formControlProperties.dependencyQuestionType : '',
        dependencyCreation: formControlProperties ? formControlProperties.dependencyCreation : '',
        dependencyMapping: formControlProperties ? formControlProperties.dependencyMapping : '',
    });

    const selectRequiredCheckBoxValue = (e, setFieldValue) => {
        setFieldValue('isRequired', e.target.checked);
        setFieldValue('requiredMessage', '');
    }

    const selectIsDefaultCheckBoxValue = (e, values, setFieldValue, index) => {
        if (values.isMulti) {
            setFieldValue(`optionsData.${index}.isdefault`, e.target.checked);
        }
        else {
            let optionsData = values.optionsData.map((x, ind) => {
                if (ind !== index)
                    return { ...x, isdefault: false }
                else return { ...x, isdefault: true }
            })
            setFieldValue('optionsData', optionsData);
        }
    };

    const deleteOptions = (remove, index, values, setFieldValue) => {
        remove(index);

        if (values.dependencyCreation && values.optionsData?.[index]?.label === values.dependencyCreation?.label) {
            setFieldValue('dependencyCreation', '');
        }
    }

    const showDuplicateLabelsAlert = () => {
        dispatch(
            alertActionRequest({
                message: 'In Options Label or values duplicates are present Please remove them',
                status: false,
                tranId: Date.now(),
                transKey: ''
            }));
    }

    const handleDependencyQuestionRequied = (e, setFieldValue) => {
        setFieldValue('isDependencyQuestionRequired', e.target.checked);
        if (!e.target.checked) {
            setFieldValue('dependencyQuestionType', '');
            setFieldValue('dependencyCreation', '');
        }
    }

    const handleDependencyQuestionType = (e, setFieldValue) => {
        setFieldValue('dependencyQuestionType', e.target.value);
        if (e.target.value !== 'custome')
            setFieldValue('dependencyCreation', e.target.value);
        else setFieldValue('dependencyCreation', '');
    }


    const handleRadioButtonCloseChanges = () => {
        if (index !== -1 && !formControlProperties) {
            if (digitalFormConfigurations?.isStepperForm) {
                let stepperForm = digitalStepperForms[stepperFormIndex]
                stepperForm.stepperFormControls?.splice(index, 1);
                digitalStepperForms?.splice(stepperFormIndex, 1, stepperForm)
            } else {
                formControlsList?.splice(index, 1);
            }
            let formData = {
                ...digitalFormCreationData,
                ...(digitalFormConfigurations?.isStepperForm ?
                    { digitalStepperForms: [...digitalStepperForms] } :
                    { digitalFormControls: [...formControlsList] }),
            }
            dispatch(addOrEditOrRemoveDigitalFormControlProperties(formData, EOprationalActions.EDIT))
        } else {
            dispatch(addOrEditOrRemoveDigitalFormControlProperties(digitalFormCreationData, EOprationalActions.EDIT))
        }
    }

    const openConfirmationAlert = () => {

        let optionsData = [
            {
                title: 'Yes',
                loading: 0,
                className: 'btn-info',
                function: handleRadioButtonCloseChanges,
                actionType: EConfirmationActionType.COMPONENT
            },
            {
                title: 'No',
                loading: 0,
                className: 'btn-danger'
            }
        ] as IConfirmOptions[];
        let confirmModel = {
            title: 'Do You Want to Discard the Changes ?',
            subtitle: ``,
            options: optionsData,
            transKey: '1234'
        } as IConfirmModel;
        dispatch(setConfirmationOpen(confirmModel));
    }

    return (
        <Formik
            enableReinitialize
            initialValues={radioButtonsInitailValues()}
            validationSchema={radioButtonValidationSchema}
            onSubmit={values => {
                let duplicatedLabelsAndValues: any[] = [];
                values.optionsData.forEach((x, index, a) => {
                    let ind = a.findIndex(y => y.label === x.label);
                    if (ind === index) {
                        let ind = a.findIndex(y => y.value === x.value)
                        if (ind !== index) {
                            duplicatedLabelsAndValues.push(x as any);
                        }
                    } else duplicatedLabelsAndValues.push(x as any)
                })
                if (duplicatedLabelsAndValues.length > 0)
                    showDuplicateLabelsAlert();
                else {
                    const { formControlLabel, ...properties } = values
                    if (index !== -1) {
                        let formControl = formControlsList[index]
                        formControl.formControlProperties = properties;
                        formControl.formControlLabel = formControlLabel;
                        if (digitalFormConfigurations?.isStepperForm) {
                            formControl.stepperFormId = digitalStepperForms[stepperFormIndex]?.stepperFormId
                        }
                        formControlsList?.splice(index, 1, formControl);
                    }
                    if (digitalFormConfigurations?.isStepperForm) {
                        let stepperForm = digitalStepperForms[stepperFormIndex]
                        stepperForm.stepperFormControls = [...formControlsList];
                        digitalStepperForms?.splice(stepperFormIndex, 1, stepperForm)
                    }
                    let formData = {
                        ...digitalFormCreationData,
                        ...(digitalFormConfigurations?.isStepperForm ?
                            { digitalStepperForms: [...digitalStepperForms] } :
                            { digitalFormControls: [...formControlsList] }),
                    }
                    dispatch(addOrEditOrRemoveDigitalFormControlProperties(formData, EOprationalActions.EDIT))

                    console.log('onSubmit==>', values)
                }
            }}
        >
            {
                ({ values, setFieldValue, errors, touched, setFieldTouched, submitForm }) => {
                    return <Form className='form-elements'>
                        <div>
                            <h6>{`${formControlType} Properties`}</h6>
                            <Row>
                                <Col sm="12">
                                    <FormGroup>
                                        <Field id="formControlLabel" type="text" placeholder="Enter Label" name="formControlLabel" className="form-control" />
                                        <ErrorMessage name="formControlLabel" component="div" className="text-danger" />
                                    </FormGroup>
                                </Col>
                            </Row>
                            <hr />
                        </div>
                        <div className="form-inputs">
                            <Row>
                                <Col sm='12'>
                                    <FormGroup>
                                        <Label htmlFor='tag'>Tag Name </Label>
                                        <Field id='tag' type='text' name='tag' placeholder='Enter Tag Name' className='form-control' />
                                        <ErrorMessage name='tag' component='div' className='text-danger' />
                                    </FormGroup>
                                </Col>
                                <Col sm="12" className="d-flex my-2">
                                    <FormGroup className='custom-control custom-checkbox mb-0 pl-0'>
                                        <Field id="isRequired" type="checkbox" name="isRequired" className="custom-control-input" onChange={(e) => selectRequiredCheckBoxValue(e, setFieldValue)} />
                                        <label className="custom-control-label" htmlFor="isRequired">Is Required :</label>
                                    </FormGroup>
                                </Col>
                                {values.isRequired && <Col sm='12'>
                                    <FormGroup>
                                        <Label htmlFor='requiredMessage'>Required Message </Label>
                                        <Field id='requiredMessage' type='text' name='requiredMessage' placeholder='Enter your Required Message' className='form-control' />
                                        <ErrorMessage name='requiredMessage' component='div' className='text-danger' />
                                    </FormGroup>
                                </Col>}
                                <Col sm="12">
                                    <FormGroup className="form-radio">
                                        <Label>Display Type :</Label>
                                        <div className="d-flex">
                                            <Field type="radio" name="displayType" value={EDisplayType.Horizontal} />
                                            <span className='ml-1 mr-3'>{EDisplayType.Horizontal}</span>
                                            <Field type="radio" name="displayType" value={EDisplayType.Vertical} />
                                            <span className='ml-1 mr-3'>{EDisplayType.Vertical}</span>
                                        </div>
                                        {/* <ErrorMessage name='dependencyQuestionType' component='div' className='text-danger' /> */}
                                    </FormGroup>
                                </Col>
                                <Col sm="12">
                                    <h6 className='font-weight-400'>Options</h6>
                                    <FieldArray
                                        name='optionsData'
                                    >
                                        {
                                            ({ push, remove }) => {
                                                return values.optionsData?.map((x, index) => {
                                                    return <React.Fragment key={index}>
                                                        <div className='form-radio-options'>
                                                            <FormGroup>
                                                                <Label htmlFor="label">Label</Label>
                                                                <Field id="label" type="text" placeholder="Enter Label" name={`optionsData.${index}.label`} className="form-control" />
                                                                <ErrorMessage name={`optionsData.${index}.label`} component='div' className='text-danger' />
                                                            </FormGroup>
                                                            <FormGroup>
                                                                <Label htmlFor="value">Value</Label>
                                                                <Field id="value" type="text" placeholder="Enter Value" name={`optionsData.${index}.value`} className="form-control" />
                                                                <ErrorMessage name={`optionsData.${index}.value`} component='div' className='text-danger' />
                                                            </FormGroup>
                                                            <div className='d-flex justify-content-between align-items-center mt-3'>
                                                                <FormGroup className='custom-control custom-checkbox mb-0 left-checkbox'>
                                                                    <Field
                                                                        id={`optionsData.${index}.isdefault`}
                                                                        type="checkbox"
                                                                        name={`optionsData.${index}.isdefault`}
                                                                        className="custom-control-input"
                                                                        onChange={(e) => selectIsDefaultCheckBoxValue(e, values, setFieldValue, index)}
                                                                    />
                                                                    <label className="custom-control-label" htmlFor={`optionsData.${index}.isdefault`}>Is Default</label>
                                                                </FormGroup>
                                                                {values.optionsData?.length !== 1 && <i className='icon-delete' onClick={() => deleteOptions(remove, index, values, setFieldValue)}></i>}
                                                            </div>
                                                        </div>
                                                        {(values.optionsData.length - 1) === index &&
                                                            <div className='form-radio-add' onClick={() => push(optionsData)}>
                                                                <i className='icon-add'></i>
                                                            </div>}
                                                    </React.Fragment>
                                                })
                                            }
                                        }
                                    </FieldArray>
                                </Col>
                                <Col sm="12" className="d-flex my-2">
                                    <FormGroup className='custom-control custom-checkbox mb-0 pl-0'>
                                        <Field id="dependencyQuestionRequired" type="checkbox" name="isDependencyQuestionRequired" className="custom-control-input" onChange={(e) => handleDependencyQuestionRequied(e, setFieldValue)} />
                                        <label className="custom-control-label" htmlFor="dependencyQuestionRequired">Is Dependency Question Need :</label>
                                    </FormGroup>
                                </Col>
                                {values.isDependencyQuestionRequired && <Col sm="12">
                                    <FormGroup className="form-radio">
                                        <Label>Dependency Creation :</Label>
                                        <div className="d-flex">
                                            <Field type="radio" name="dependencyQuestionType" value='isDefined' onChange={(e) => handleDependencyQuestionType(e, setFieldValue)} /><span className='ml-1 mr-3'>Is Defined</span>
                                            <Field type="radio" name="dependencyQuestionType" value='custome' onChange={(e) => handleDependencyQuestionType(e, setFieldValue)} /><span className='ml-1 mr-3'>Custome</span>
                                        </div>
                                        <ErrorMessage name='dependencyQuestionType' component='div' className='text-danger' />
                                    </FormGroup>
                                </Col>}
                                {values.dependencyQuestionType === 'custome' && <Col sm='12'>
                                    <FormGroup>
                                        <Label> Enter Condition Value </Label>
                                        <MySelect
                                            allOption={{
                                                label: "Select All",
                                                value: 0
                                            }}
                                            isMulti={false}
                                            name='dependencyCreation'
                                            placeholder='Select Dependency Value'
                                            options={values.optionsData?.filter(x => x.label) || []}
                                            value={values.dependencyCreation || ""}
                                            onChange={(e) => { setFieldValue('dependencyCreation', e || '') }}
                                            // getOptionLabel={option => option.label}
                                            // getOptionValue={option => option.value}
                                            valueKey="value"
                                            // components={{ Option, ValueContainer }}
                                            hideSelectedOptions={false}
                                            removeSelected={false}
                                            closeMenuOnSelect={true}
                                            backspaceRemovesValue={false}
                                            isSearchable={true}
                                            allowSelectAll={false}
                                            onBlur={() => setFieldTouched('dependencyCreation', true)}
                                            noOptionsMessage={() => 'No Data Found'}
                                            className="select-box"
                                        />
                                        {errors.dependencyCreation && touched.dependencyCreation && <div className='text-danger'>{errors.dependencyCreation}</div>}
                                    </FormGroup>
                                </Col>}
                                <Col sm="12">
                                    <FormGroup>
                                        <Label htmlFor='dependencyMapping'>Dependency Mapping</Label>
                                        <MySelect
                                            allOption={{
                                                label: "Select All",
                                                value: 0
                                            }}
                                            isMulti={false}
                                            name='dependencyMapping'
                                            id='dependencyMapping'
                                            placeholder='Select Mapping Parent'
                                            options={mappingFormBuilderData || []}
                                            value={mappingFormBuilderData?.find(x => x.formControlId === values.dependencyMapping) || ""}
                                            onChange={(e) => { setFieldValue('dependencyMapping', e?.formControlId || '') }}
                                            getOptionLabel={option => option.formControlLabel}
                                            getOptionValue={option => option.formControlId}
                                            valueKey="value"
                                            hideSelectedOptions={false}
                                            removeSelected={false}
                                            closeMenuOnSelect={true}
                                            backspaceRemovesValue={false}
                                            isSearchable={true}
                                            allowSelectAll={false}
                                            onBlur={() => setFieldTouched('dependencyMapping', true)}
                                            noOptionsMessage={() => 'No Data Found'}
                                            className="select-box"
                                        />
                                    </FormGroup>
                                </Col>
                            </Row>
                        </div>
                        <div className='py-2'>
                            <button className='button button-clear mr-2' type='button' onClick={() => openConfirmationAlert()}>Close</button>
                            <button className='button button-save-bordered' type='submit'>Save</button>
                        </div>
                    </Form>
                }
            }
        </Formik>
    )
}

export default React.memo(RadioButtonPropertiesComp)