import React from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { useFormikContext, Field, Form, Formik, FieldArray, ErrorMessage } from 'formik';
import { FormGroup, Label, Row, Col } from 'reactstrap';
import * as Yup from 'yup';
import { useTranslation } from 'react-i18next';
import { customContentValidation, defultContentValidate, MySelect } from '../../../helpers/helpersIndex';
import * as _ from 'lodash';
import { IFormBuilderReducer } from 'react-redux';
import { addOrEditOrRemoveDigitalFormControlProperties, setConfirmationOpen } from '../../../store/actions';
import { EOprationalActions, EConfirmationActionType, IConfirmOptions, IConfirmModel } from '../../../models/utilitiesModel';

const TextAreaPropertiesComp: React.FC<any> = ({ arrayHelperMethods }) => {

    const { t } = useTranslation('translations');
    const dispatch = useDispatch();

    const actionData: any = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.actionData);
    const digitalFormCreationData: any = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.digitalFormCreationData);
    const stepperFormIndex: any = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.stepperFormIndex);

    const { formControlProperties, formControlLabel, formControlType, formControlId } = actionData;
    let { digitalFormControls, digitalStepperForms, digitalFormConfigurations } = digitalFormCreationData;
    let formControlsList = digitalFormConfigurations?.isStepperForm ? digitalStepperForms?.[stepperFormIndex]?.stepperFormControls : digitalFormControls;
    let index = formControlsList?.findIndex(x => x.formControlId === formControlId);
    let mappingFormBuilderData = formControlsList?.slice(0, index)?.filter(x => x.formControlProperties?.isDependencyQuestionRequired);
    if (digitalFormConfigurations?.isStepperForm) {
        let stepperDependencyControlsList = digitalStepperForms?.slice(0, stepperFormIndex)?.flatMap(y => y.stepperFormControls)?.filter(x => x.formControlProperties?.isDependencyQuestionRequired);
        mappingFormBuilderData = [...stepperDependencyControlsList, ...mappingFormBuilderData]
    }


    const textareaInitialValues = () => ({
        formControlLabel: formControlLabel ? formControlLabel : '',
        tag: formControlProperties ? formControlProperties?.tag : '',
        min: formControlProperties ? formControlProperties.min : 1,
        max: formControlProperties ? formControlProperties.max : '',
        isRequired: formControlProperties ? formControlProperties.isRequired : false,
        requiredMessage: formControlProperties ? formControlProperties.requiredMessage : '',
        defaultValue: formControlProperties ? formControlProperties.defaultValue : '',
        // isDependencyQuestionRequired: formControlProperties ? formControlProperties.isDependencyQuestionRequired : false,
        // dependencyQuestionType: formControlProperties ? formControlProperties.dependencyQuestionType : '',
        // dependencyCreation: formControlProperties ? formControlProperties.dependencyCreation : '',
        // dependencyMapping: formControlProperties ? formControlProperties.dependencyMapping : '',
    });

    const textareaValidationSchema = Yup.object().shape({
        formControlLabel: customContentValidation(t, t('controleErrors.required'), { patternType: 'alphanumericspacesp', message: 'alphanumaricandspace', spacialChar: '' }, 255, 1),
        max: customContentValidation(t, t('controleErrors.required'), { patternType: 'number', message: 'number', spacialChar: '' }, 255, 1),
        requiredMessage: Yup.string().when('isRequired', {
            is: (isRequired) => isRequired,
            then: customContentValidation(t, t('controleErrors.required'), { patternType: 'alphanumericspacesp', message: 'alphanumaricandspace', spacialChar: '' }, 255, 1),
            otherWise: defultContentValidate(t('controleErrors.required'))
        }),
        defaultValue: Yup.lazy((val) => {
            if (val)
                return customContentValidation(t, t('controleErrors.required'), { patternType: 'alphanumericspacesp', message: 'alphanumaricandspace', spacialChar: '' }, 255, 1)
            else return defultContentValidate('')
        }),
        // dependencyQuestionType: Yup.string().when('isDependencyQuestionRequired', {
        //     is: (isDependencyQuestionRequired) => isDependencyQuestionRequired,
        //     then: defultContentValidate(t('controleErrors.required')),
        //     otherWise: defultContentValidate('')
        // }),
        // dependencyCreation: Yup.string().when('dependencyQuestionType', {
        //     is: (dependencyQuestionType) => dependencyQuestionType === 'custome',
        //     then: customContentValidation(t, t('controleErrors.required'), { patternType: 'alphanumericspacesp', message: 'alphanumaricandspace', spacialChar: '' }, 255, 1),
        //     otherWise: defultContentValidate('')
        // }),
    })

    const selectRequiredCheckBoxValue = (e, setFieldValue) => {
        setFieldValue('isRequired', e.target.checked);
        setFieldValue('requiredMessage', '');
    }

    const handleDependencyQuestionRequied = (e, setFieldValue) => {
        setFieldValue('isDependencyQuestionRequired', e.target.checked);
        if (!e.target.checked) {
            setFieldValue('dependencyQuestionType', '');
            setFieldValue('dependencyCreation', '');
        }
    }

    const handleDependencyQuestionType = (e, setFieldValue) => {
        setFieldValue('dependencyQuestionType', e.target.value);
        if (e.target.value !== 'custome')
            setFieldValue('dependencyCreation', e.target.value);
        else setFieldValue('dependencyCreation', '');

    }

    const handleTextAreaCloseChanges = () => {
        if (index !== -1 && !formControlProperties) {
            if (digitalFormConfigurations?.isStepperForm) {
                let stepperForm = digitalStepperForms[stepperFormIndex]
                stepperForm.stepperFormControls?.splice(index, 1);
                digitalStepperForms?.splice(stepperFormIndex, 1, stepperForm)
            } else {
                formControlsList?.splice(index, 1);
            }

            let formData = {
                ...digitalFormCreationData,
                ...(digitalFormConfigurations?.isStepperForm ?
                    { digitalStepperForms: [...digitalStepperForms] } :
                    { digitalFormControls: [...formControlsList] }),
            }
            dispatch(addOrEditOrRemoveDigitalFormControlProperties(formData, EOprationalActions.EDIT))
        } else {
            dispatch(addOrEditOrRemoveDigitalFormControlProperties(digitalFormCreationData, EOprationalActions.EDIT))
        }
    }

    const openConfirmationAlert = () => {

        let optionsData = [
            {
                title: 'Yes',
                loading: 0,
                className: 'btn-info',
                function: handleTextAreaCloseChanges,
                actionType: EConfirmationActionType.COMPONENT
            },
            {
                title: 'No',
                loading: 0,
                className: 'btn-danger'
            }
        ] as IConfirmOptions[];
        let confirmModel = {
            title: 'Do You Want to Discard the Changes?',
            subtitle: ``,
            options: optionsData,
            transKey: '1234'
        } as IConfirmModel;
        dispatch(setConfirmationOpen(confirmModel));
    }
    return (
        <Formik
            enableReinitialize
            initialValues={textareaInitialValues()}
            validationSchema={textareaValidationSchema}
            onSubmit={(values) => {
                const { formControlLabel, ...properties } = values
                if (index !== -1) {
                    let formControl = formControlsList[index]
                    formControl.formControlProperties = properties;
                    formControl.formControlLabel = formControlLabel;
                    if (digitalFormConfigurations?.isStepperForm) {
                        formControl.stepperFormId = digitalStepperForms[stepperFormIndex]?.stepperFormId
                    }
                    formControlsList?.splice(index, 1, formControl);
                }
                if (digitalFormConfigurations?.isStepperForm) {
                    let stepperForm = digitalStepperForms[stepperFormIndex]
                    stepperForm.stepperFormControls = [...formControlsList];
                    digitalStepperForms?.splice(stepperFormIndex, 1, stepperForm)
                }
                let formData = {
                    ...digitalFormCreationData,
                    ...(digitalFormConfigurations?.isStepperForm ?
                        { digitalStepperForms: [...digitalStepperForms] } :
                        { digitalFormControls: [...formControlsList] }),
                }
                dispatch(addOrEditOrRemoveDigitalFormControlProperties(formData, EOprationalActions.EDIT))
            }}
        >
            {
                ({ values, setFieldValue, setFieldTouched, submitForm }) => {
                    return <Form className='form-elements'>
                        <div>
                            <h6>{`${formControlType} Properties`}</h6>
                            <Row>
                                <Col sm="12">
                                    <FormGroup>
                                        {/* <Label htmlFor="formControlLabel">Label</Label> */}
                                        <Field id="formControlLabel" type="text" placeholder="Enter Label" name="formControlLabel" className="form-control" />
                                        <ErrorMessage name="formControlLabel" component="div" className="text-danger" />
                                    </FormGroup>
                                </Col>
                            </Row>
                            <hr />
                        </div>
                        <div className="form-inputs">
                            <Row>
                                <Col sm="12">
                                    <FormGroup>
                                        <Label htmlFor="tag">Tag Name </Label>
                                        <Field id="tag" type='text' name='tag' placeholder='Enter Tag Name' className='form-control' />
                                        <ErrorMessage name='tag' component='div' className='text-danger' />
                                    </FormGroup>
                                </Col>
                                <Col sm="12">
                                    <FormGroup>
                                        <Label htmlFor="minLength">Min Length </Label>
                                        <Field id="minLength" type='number' name='min' placeholder='Enter Min Length' className='form-control' />
                                        <ErrorMessage name='min' component='div' className='text-danger' />
                                    </FormGroup>
                                </Col>
                                <Col sm="12">
                                    <FormGroup>
                                        <Label htmlFor="maxLength">Max Length </Label>
                                        <Field id="maxLength" type='number' name='max' placeholder='Enter Max Length' className='form-control' />
                                        <ErrorMessage name='max' component='div' className='text-danger' />
                                    </FormGroup>
                                </Col>
                                {/* <Col sm="12">
                                    <FormGroup>
                                        <Label htmlFor="pattern">Pattern </Label>
                                        <Field id="pattern" type='text' name='pattern' placeholder='Enter Your Pattern' className='form-control' />
                                    </FormGroup>
                                </Col> */}
                                <Col sm="12">
                                    <FormGroup>
                                        <Label htmlFor="defaultValue">Default Value </Label>
                                        <Field id="defaultValue" type='text' name='defaultValue' placeholder='Enter your Default Value' className='form-control' />
                                        <ErrorMessage name='defaultValue' component='div' className='text-danger' />
                                    </FormGroup>
                                </Col>
                                <Col sm="12" className="d-flex my-2">
                                    <FormGroup className='custom-control custom-checkbox mb-0 pl-0'>
                                        <Field id="isRequired" type="checkbox" name="isRequired" className="custom-control-input" onChange={(e) => selectRequiredCheckBoxValue(e, setFieldValue)} />
                                        <label className="custom-control-label" htmlFor="isRequired">Is Required :</label>
                                    </FormGroup>
                                </Col>
                                {values.isRequired && <Col sm='12'>
                                    <FormGroup>
                                        <Label label='requiredMessage'>Required Message </Label>
                                        <Field id='requiredMessage' type='text' name='requiredMessage' placeholder='Enter your Required Message' className='form-control' />
                                        <ErrorMessage name='requiredMessage' component='div' className='text-danger' />
                                    </FormGroup>
                                </Col>}
                                {/* <Col sm="12" className="d-flex my-2">
                                    <FormGroup className='custom-control custom-checkbox mb-0 pl-0'>
                                        <Field id="dependencyQuestionRequired" type="checkbox" name="isDependencyQuestionRequired" className="custom-control-input" onChange={(e) => handleDependencyQuestionRequied(e, setFieldValue)} />
                                        <label className="custom-control-label" htmlFor="dependencyQuestionRequired">Is Dependency Question Need :</label>
                                    </FormGroup>
                                </Col> */}
                                {/* {values.isDependencyQuestionRequired && <Col sm="12">
                                    <FormGroup className="form-radio">
                                        <Label>Dependency Creation :</Label>
                                        <div className="d-flex">
                                            <Field type="radio" name="dependencyQuestionType" value='isDefined' onChange={(e) => handleDependencyQuestionType(e, setFieldValue)} /><span className='ml-1 mr-3'>Is Defined</span>
                                            <Field type="radio" name="dependencyQuestionType" value='custome' onChange={(e) => handleDependencyQuestionType(e, setFieldValue)} /><span className='ml-1 mr-3'>Custome</span>
                                        </div>
                                        <ErrorMessage name='dependencyQuestionType' component='div' className='text-danger' />
                                    </FormGroup>
                                </Col>} */}
                                {/* {values.dependencyQuestionType === 'custome' && <Col sm='12'>
                                    <FormGroup>
                                        <Label htmlFor='conditionValue'> Enter Condition Value </Label>
                                        <Field id='conditionValue' type='text' name='dependencyCreation' placeholder='Enter Dependency Creation Value' className='form-control' />
                                        <ErrorMessage name='dependencyCreation' component='div' className='text-danger' />
                                    </FormGroup>
                                </Col>} */}
                                {/* <Col sm="12">
                                    <FormGroup>
                                        <Label htmlFor='dependencyMapping'>Dependency Mapping</Label>
                                        <MySelect
                                            allOption={{
                                                label: "Select All",
                                                value: 0
                                            }}
                                            isMulti={false}
                                            name='dependencyMapping'
                                            id='dependencyMapping'
                                            placeholder='Select Mapping Parent'
                                            options={mappingFormBuilderData || []}
                                            value={mappingFormBuilderData?.find(x => x.formControlId === values.dependencyMapping) || ""}
                                            onChange={(e) => { setFieldValue('dependencyMapping', e?.formControlId || '') }}
                                            getOptionLabel={option => option.formControlLabel}
                                            getOptionValue={option => option.formControlId}
                                            valueKey="value"
                                            hideSelectedOptions={false}
                                            removeSelected={false}
                                            closeMenuOnSelect={true}
                                            backspaceRemovesValue={false}
                                            isSearchable={true}
                                            allowSelectAll={false}
                                            onBlur={() => setFieldTouched('dependencyMapping', true)}
                                            noOptionsMessage={() => 'No Data Found'}
                                            className="select-box"
                                        />
                                    </FormGroup>
                                </Col> */}
                            </Row>
                        </div>
                        <div className='py-2'>
                            <button className='button button-clear mr-2' type='button' onClick={() => openConfirmationAlert()}>Close</button>
                            <button className='button button-save-bordered' type='submit'>Save</button>
                        </div>
                    </Form>
                }
            }
        </Formik>
    )
}

export default React.memo(TextAreaPropertiesComp)