import React, { useEffect, useState } from 'react';
import { useLocation } from "react-router-dom";
import { useSelector, useDispatch } from 'react-redux';
import { createInputdataOfFormBuilderRequest, getFormByFormIdFormBuildersRequest, setClearFormByIdData } from '../../store/actions';
import { IFormBuilderModel, IFormBuilderReducer } from './digitalFormModel';
import LayoutAlertMessage from 'components/Layout/LayoutAlertMessage';
import { EAPPModules, EResponseStatus } from '../../models/utilitiesModel';
import { useTranslation } from 'react-i18next';
import DigitalFormBuilderRenderContainer from './digitalFormBuilderRenderContainer';
import DigitalStepperFormRender from './digitalStepperFormRender';



const QueryForm = (props) => {
    const { t } = useTranslation('translations');
    const dispatch = useDispatch();
    const [state, setState] = useState('');
    const [WebHookStatus, setWebHookStatus] = useState('');

    const handleState = (status) => {
        setState(status);
    }

    const useQuery = () => {
        const { search } = useLocation();
        return React.useMemo(() => new URLSearchParams(search), [search]);
    }
    const id = useQuery().get('id') || '';
    useEffect(() => {
        id && dispatch(getFormByFormIdFormBuildersRequest(id))
        return () => { dispatch(setClearFormByIdData()) }
    }, []);
    const form: IFormBuilderModel | undefined = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.formDataById);

    const submitCallBack = (values) => {
        if (form?.formBuilderConfigurations?.isDigitalSave) {
            dispatch(createInputdataOfFormBuilderRequest(values, handleState));
        }
        else
            handleState(EResponseStatus.NOTEXECUTED)

        if (form?.formBuilderConfigurations?.isWebHook) {
            fetch((form?.formBuilderConfigurations?.api || ''), {
                method: 'POST',
                headers: form?.formBuilderConfigurations?.apiHeaders,
                body: JSON.stringify(values)
            })
                .then(resp => {
                    if (resp?.ok)
                        setWebHookStatus(EResponseStatus.SUCCESS)
                    else
                        setWebHookStatus(EResponseStatus.FAILED)

                    console.log('Out_Post_Response==>', resp);
                })
                .catch(err => {
                    setWebHookStatus(EResponseStatus.FAILED)
                    console.error('Out_Post_Response_Err==>', err)
                });
        } else
            setWebHookStatus(EResponseStatus.NOTEXECUTED);
        console.log(`${EAPPModules.DYNAMICFORMBUILDER}_QueryForm_submitCallBack==>`, values);
    }

    const digitalFormCreationData = {
        digitalFormTitle: form?.formBuilderTitle || "",
        digitalFormId: form?.id || '',
        digitalFormConfigurations: {
            ...form?.formBuilderConfigurations
        },
        digitalFormControls: !form?.formBuilderConfigurations?.isStepperForm ? form?.formBuilderData : [],
        digitalStepperForms: form?.formBuilderConfigurations?.isStepperForm ? form.formBuilderData : [],
    }

    console.log(`${EAPPModules.DYNAMICFORMBUILDER}_QueryForm==>`, { id, props, sample: form, handleState });

    const getStatusAndMessage = () => {
        if (state && WebHookStatus) {
            if (state === EResponseStatus.NOTEXECUTED && WebHookStatus === EResponseStatus.NOTEXECUTED) return '';
            else if (state === EResponseStatus.FAILED && WebHookStatus === EResponseStatus.FAILED) return <div>Data Insertion Failed</div>
            else if (state === EResponseStatus.SUCCESS || WebHookStatus === EResponseStatus.SUCCESS) return <div>
                <div>Data Inserted Successfully</div>
                {state === EResponseStatus.FAILED && <small>Note : Digital Save Failed</small>}
                {WebHookStatus === EResponseStatus.FAILED && <small>Note : WebHook Save Failed</small>}
            </div>
            else if (state === EResponseStatus.FAILED || WebHookStatus === EResponseStatus.FAILED) return <div>Data Insertion Failed</div>
        }
        return ''
    }

    return (<div>
        {getStatusAndMessage() ? getStatusAndMessage() : <>
            <LayoutAlertMessage />
            {Boolean(form) && <>
                {/* <h6>{form?.formBuilderTitle}</h6> */}
                {!form?.formBuilderConfigurations?.isStepperForm && <DigitalFormBuilderRenderContainer digitalFormCreationData={digitalFormCreationData} submitCallBack={submitCallBack} />}
                {form?.formBuilderConfigurations?.isStepperForm && <DigitalStepperFormRender digitalFormCreationData={digitalFormCreationData} submitCallBack={submitCallBack} />}

            </>}
            {Boolean(id) && !Boolean(form) && <div style={{ textAlign: "center" }}>No Form found for Id</div>}
            {!Boolean(id) && <div style={{ textAlign: "center" }}>invalid Id</div>}
        </>}
    </div >)
}
export default React.memo(QueryForm);