import { EAPPModules } from '../../models/utilitiesModel';
import React, { useEffect, useState } from 'react';
import { useTranslation } from 'react-i18next';
import { useDispatch, useSelector } from 'react-redux';
import { createInputdataOfFormBuilderRequest, getFormByFormIdFormBuildersRequest, setClearFormByIdData } from '../../store/actions';
import { IFormBuilderModel, IFormBuilderReducer } from './digitalFormModel';
import DigitalFormBuilderRenderContainer from './digitalFormBuilderRenderContainer';
import DigitalStepperFormRender from './digitalStepperFormRender';



const RenderForm = (props) => {
    const dispatch = useDispatch();
    const { t } = useTranslation('translations');
    const { id, submitCallBack, cancelCallBack } = props;
    const [state, setState] = useState(false);

    const handleState = (status) => {
        setState(status);
    }

    useEffect(() => {
        if (id) {
            dispatch(getFormByFormIdFormBuildersRequest(id));
        }
        return () => { dispatch(setClearFormByIdData()) }
    }, [id]);

    const form: IFormBuilderModel | undefined = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.formDataById);
    console.log(`${EAPPModules.DYNAMICFORMBUILDER}_RenderForm==>`, { props, sample: form });

    const handleSubmit = (values) => {
        if (form?.formBuilderConfigurations?.isDigitalSave) {
            dispatch(createInputdataOfFormBuilderRequest(values, handleState));
        } else if (submitCallBack) {
            submitCallBack(values);
        }
        console.log(`${EAPPModules.DYNAMICFORMBUILDER}_RenderForm_submitCallBack==>`, values);
    }

    const digitalFormCreationData = {
        digitalFormTitle: form?.formBuilderTitle || "",
        digitalFormId: form?.id || '',
        digitalFormConfigurations: {
            ...form?.formBuilderConfigurations
        },
        digitalFormControls: !form?.formBuilderConfigurations?.isStepperForm ? form?.formBuilderData : [],
        digitalStepperForms: form?.formBuilderConfigurations?.isStepperForm ? form.formBuilderData : [],
    }
    return (<div>
        {state ? t('DynamicFormBuilder.inputDataSuccess') : <>
            {Boolean(form) &&
                <>
                    {/* <h6>{form?.formBuilderTitle}</h6> */}
                    {!form?.formBuilderConfigurations?.isStepperForm && <DigitalFormBuilderRenderContainer digitalFormCreationData={digitalFormCreationData} submitCallBack={handleSubmit} cancelCallBack={cancelCallBack} />}
                    {form?.formBuilderConfigurations?.isStepperForm && <DigitalStepperFormRender digitalFormCreationData={digitalFormCreationData} submitCallBack={handleSubmit} cancelCallBack={cancelCallBack} />}
                </>}
            {Boolean(id) && !Boolean(form) && <div style={{ textAlign: "center", marginTop: "100px" }}>No Form found for Id</div>}
            {!Boolean(id) && <div style={{ textAlign: "center" }}>invalid Id</div>}
        </>}
    </div >)
}
export default React.memo(RenderForm);