import React from 'react';
import { Formik, Field, Form, ErrorMessage } from 'formik';
import { Row, Col, FormGroup, Label } from 'reactstrap';
import { MySelect, customContentValidation, Option, ValueContainer, defultContentValidate } from '../../helpers/helpersIndex';
import DatePicker from "react-datepicker";
import { useDispatch, useSelector } from 'react-redux';
import { setResetDigitalFormConfigurationsModel, addOrEditOrRemoveDigitalFormControlProperties, setResetDigitalFormPriviewModel, setDigitalStepperFormAndIndex, resetCancelDigitalFormData, createOrUpdateFormFormBuildersRequest } from '../../store/actions';
import DigitalFormConfigurationModel from './digitalFormConfigurationsModel';
import * as Yup from 'yup';
import { v4 as uuidv4 } from 'uuid';
import { useTranslation } from 'react-i18next';
import { IFormBuilderReducer, EElementInputTypes } from './digitalFormModel';
import { EOprationalActions } from '../../models/utilitiesModel';


function DigitalFormAction() {

    const dispatch = useDispatch();
    const { t } = useTranslation('translations');

    const digitalFormCreationData: any = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.digitalFormCreationData);
    const stepperFormIndex: any = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.stepperFormIndex);

    const { digitalFormConfigurations, digitalFormControls, digitalStepperForms, digitalFormTitle, digitalFormId } = digitalFormCreationData || {};

    const digitalFormRenderData = digitalFormConfigurations?.isStepperForm ? [...digitalStepperForms?.[stepperFormIndex]?.stepperFormControls] : digitalFormControls;


    console.log('DigitalFormAction=>', digitalFormCreationData)

    const handleConfigurationModel = () => {
        dispatch(setResetDigitalFormConfigurationsModel(true))
    }

    const handlePriviewModel = () => {
        dispatch(setResetDigitalFormPriviewModel(true))
    }

    const handleFormControlEditDelete = (formData, type) => {
        if (type === EOprationalActions.EDIT)
            dispatch(addOrEditOrRemoveDigitalFormControlProperties(formData, EOprationalActions.ADD))
        else
            dispatch(addOrEditOrRemoveDigitalFormControlProperties(formData, EOprationalActions.DELETE))
    }

    const handleDigitalFormStepper = (type, { setFieldValue, setFieldTouched, values, setValues }) => {
        if (type === 'next') {
            if (digitalStepperForms[stepperFormIndex]?.stepperFormControls?.length === 0) {
                setFieldValue('error', 'Add At least one control to Move Next/Previous');
                setFieldTouched('digitalStepperFormTitle', true)
            } else if (digitalStepperForms[stepperFormIndex]?.stepperFormControls?.length > 0 && values.digitalStepperFormTitle && stepperFormIndex === (digitalStepperForms?.length - 1)) {
                let formData = {
                    stepperFormId: uuidv4(),
                    stepperFormName: '',
                    stepperFormControls: []
                }
                digitalStepperForms[stepperFormIndex].stepperFormName = values.digitalStepperFormTitle;
                let stepperFormsData = {
                    ...digitalFormCreationData,
                    digitalFormTitle: values.digitalFormTitle,
                    digitalStepperForms: [...(digitalStepperForms || []), formData]
                }
                dispatch(setDigitalStepperFormAndIndex(stepperFormsData, (stepperFormIndex + 1)));
                setFieldValue('error', '');
                setFieldValue('digitalStepperFormTitle', '');
                setFieldTouched('digitalStepperFormTitle', false);
            } else if (values.digitalStepperFormTitle) {
                dispatch(setDigitalStepperFormAndIndex(digitalFormCreationData, (stepperFormIndex + 1)));
            } else if (!values.digitalStepperFormTitle) {
                setFieldTouched('digitalStepperFormTitle', true);
            }
        } else if (type === 'previous' || type === 'priview') {
            if (digitalStepperForms[stepperFormIndex]?.stepperFormControls?.length === 0) {
                setFieldValue('error', 'Add At least one control to Move Next/Previous');
                setFieldTouched('digitalStepperFormTitle', true)
            } else if (digitalStepperForms[stepperFormIndex]?.stepperFormControls?.length > 0 && values.digitalStepperFormTitle) {
                digitalStepperForms[stepperFormIndex].stepperFormName = values.digitalStepperFormTitle;
                let stepperFormsData = {
                    ...digitalFormCreationData,
                    digitalFormTitle: values.digitalFormTitle,
                    digitalStepperForms: [...(digitalStepperForms || [])]
                }
                if (type === 'previous')
                    dispatch(setDigitalStepperFormAndIndex(stepperFormsData, (stepperFormIndex - (stepperFormIndex > 0 ? 1 : 0))));
                else {
                    dispatch(setDigitalStepperFormAndIndex(stepperFormsData, stepperFormIndex));
                    dispatch(setResetDigitalFormPriviewModel(true))
                }
            } else if (!values.digitalStepperFormTitle) {
                setFieldTouched('digitalStepperFormTitle', true);
            }
        }
    }

    const handleDeleteDigitalStepperForm = (setFieldValue) => {
        let StepperformData = digitalStepperForms;
        if (digitalStepperForms?.length !== 1) {
            StepperformData?.splice(stepperFormIndex, 1);
        } else {
            let formData = {
                stepperFormId: uuidv4(),
                stepperFormName: '',
                stepperFormControls: []
            }
            StepperformData = [formData];
            setFieldValue('digitalStepperFormTitle', '')
        }
        let digitalFormData = {
            ...digitalFormCreationData,
            digitalStepperForms: [...(StepperformData || [])]
        }
        dispatch(setDigitalStepperFormAndIndex(digitalFormData, (stepperFormIndex - (stepperFormIndex > 0 ? 1 : 0))));
    }

    const handleResetForm = (resetForm) => {
        resetForm()
        const digitalFormCreationData = {
            digitalFormTitle: "",
            digitalFormId: uuidv4(),
            digitalFormConfigurations: {
                noOfColumns: 2,
                isDigitalSave: false,
                isStepperForm: false,
                selectedTheme: 'theme-1',
                isWebHook: false
            },
            digitalFormControls: [],
            digitalStepperForms: []
        }
        dispatch(resetCancelDigitalFormData(EOprationalActions.ADD, digitalFormCreationData))
    }

    const handleFormCancel = () => {
        dispatch(resetCancelDigitalFormData(EOprationalActions.UNSELECT, null))
    }

    const enableorDisableButtons = !digitalFormConfigurations?.isStepperForm ? !!(digitalFormControls?.[digitalFormControls?.length - 1]?.formControlProperties) :
        !!(digitalStepperForms[digitalStepperForms?.length - 1]?.stepperFormControls[digitalStepperForms[digitalStepperForms?.length - 1]?.stepperFormControls?.length - 1]?.formControlProperties)

    return (
        <Formik
            enableReinitialize
            initialValues={{
                digitalFormTitle: digitalFormTitle ? digitalFormTitle : "",
                id: digitalFormId ? digitalFormId : '',
                digitalStepperFormTitle: digitalStepperForms ? (digitalStepperForms[stepperFormIndex]?.stepperFormName || '') : '',
                error: ''
            }}
            validationSchema={Yup.object().shape({
                digitalFormTitle: customContentValidation(t, t('controleErrors.required'), { patternType: 'alphanumericspacesp', message: 'alphanumaricandspace', spacialChar: '' } as any, 255, 1),
                digitalStepperFormTitle: Yup.lazy(() => {
                    if (digitalFormConfigurations?.isStepperForm) {
                        return customContentValidation(t, t('controleErrors.required'), { patternType: 'alphanumericspacesp', message: 'alphanumaricandspace', spacialChar: '' } as any, 255, 1)
                    } else {
                        return defultContentValidate('')
                    }
                })
            })}
            onSubmit={(values, { resetForm }) => {
                console.log('values==>', values);
                // alert(JSON.stringify(digitalFormCreationData, null, 6))
                if (digitalFormConfigurations?.isStepperForm) {
                    digitalStepperForms[stepperFormIndex].stepperFormName = values.digitalStepperFormTitle;
                }
                dispatch(createOrUpdateFormFormBuildersRequest({
                    formBuilderTitle: values.digitalFormTitle,
                    id: values.id || '',
                    formBuilderData: digitalFormConfigurations?.isStepperForm ? digitalStepperForms : digitalFormControls,
                    formBuilderConfigurations: digitalFormConfigurations
                }, null));
                // resetForm();
            }}
        >
            {
                ({ values, setFieldValue, setFieldTouched, setValues, resetForm }) => {
                    return <Form className={'formCreation-left-rgt mbfcl-rgt'}>
                        <div className='form-settings'>
                            <Row className="mx-0">
                                <Col sm="4" md="6" lg="4" xl="4">
                                    <FormGroup>
                                        <Label htmlFor="digitalFormTitle">Form Name</Label>
                                        <Field id="digitalFormTitle" type="text" placeholder="Enter Form name" name="digitalFormTitle" className="form-control" />
                                        <ErrorMessage name="digitalFormTitle" component="div" className="text-danger" />
                                    </FormGroup>
                                </Col>
                                <Col sm="4" md="6" lg="4" xl="4">
                                    <FormGroup>
                                        <div className='mt-4'>
                                            <i className='icon-clarity_settings' onClick={handleConfigurationModel}></i>
                                        </div>
                                    </FormGroup>
                                </Col>

                            </Row>
                        </div>
                        {/* <div className='mb-elements' onClick={() => this.setState({ formElements: true })}> &lt;&lt;&nbsp;Elements</div> */}
                        <div className='form-adding'>
                            <div className='form-adding-element'>
                                <Row className="px-2">
                                    {digitalFormConfigurations?.isStepperForm && <Col sm="4" md="6" lg="4" xl="4">
                                        <FormGroup>
                                            <Label htmlFor="digitalStepperFormTitle">Stepper Form Name</Label>
                                            <Field id="digitalStepperFormTitle" type="text" placeholder="Enter Stepper Form name" name="digitalStepperFormTitle" className="form-control" />
                                            <ErrorMessage name="digitalStepperFormTitle" component="div" className="text-danger" />
                                        </FormGroup>
                                    </Col>}
                                    {
                                        digitalFormCreationData && digitalFormRenderData?.map((x, ind) => {
                                            const { formControlLabel, formControlType, formControlProperties } = x;
                                            return <React.Fragment key={ind}>
                                                {formControlProperties && formControlType === EElementInputTypes.SINGLELINE &&
                                                    <Col sm="6" className="hover-action">
                                                        <FormGroup>
                                                            <Label htmlFor={formControlLabel}>{formControlLabel}</Label>
                                                            <span className='float-right d-flex'>
                                                                <i className='icon-edit' onClick={() => { handleFormControlEditDelete(x, EOprationalActions.EDIT) }}></i>
                                                                <i className='icon-delete' onClick={() => { handleFormControlEditDelete(x, EOprationalActions.DELETE) }}></i>
                                                                {/* <i className='icon-Move'></i> */}
                                                            </span>
                                                            <Field id={formControlLabel} type='text' placeholder={formControlLabel} name={`formBuilderData.${ind}.formControlLabel`} className="form-control" readOnly />
                                                        </FormGroup>
                                                    </Col>
                                                }
                                                {formControlProperties && (formControlType === EElementInputTypes.TEXTAREA) &&
                                                    <Col sm="6" className="hover-action">
                                                        <FormGroup>
                                                            <Label htmlFor={formControlLabel}>{formControlLabel}</Label>
                                                            <span className='float-right d-flex'>
                                                                <i className='icon-edit' onClick={() => { handleFormControlEditDelete(x, EOprationalActions.EDIT) }}></i>
                                                                <i className='icon-delete' onClick={() => { handleFormControlEditDelete(x, EOprationalActions.DELETE) }}></i>
                                                                {/* <i className='icon-Move'></i> */}
                                                            </span>
                                                            <Field id={formControlLabel} as='textarea' placeholder={formControlLabel} name={`formBuilderData.${ind}.formControlLabel`} className="form-control" readOnly />
                                                        </FormGroup>
                                                    </Col>
                                                }
                                                {formControlProperties && formControlType === EElementInputTypes.DROPDOWN &&
                                                    <Col sm='6' className="hover-action">
                                                        <FormGroup>
                                                            <Label htmlFor={formControlLabel}>{formControlLabel}</Label>
                                                            <span className='float-right d-flex'>
                                                                <i className='icon-edit' onClick={() => { handleFormControlEditDelete(x, EOprationalActions.EDIT) }}></i>
                                                                <i className='icon-delete' onClick={() => { handleFormControlEditDelete(x, EOprationalActions.DELETE) }}></i>
                                                                {/* <i className='icon-Move'></i> */}
                                                            </span>
                                                            <MySelect
                                                                allOption={{
                                                                    label: "Select All",
                                                                    value: 0
                                                                }}
                                                                isMulti={formControlProperties?.isMulti || false}
                                                                name={formControlLabel}
                                                                placeholder={formControlLabel}
                                                                options={formControlProperties?.optionsData || []}
                                                                value={x[formControlLabel]}
                                                                onChange={(e) => { }}
                                                                getOptionLabel={option => option.label}
                                                                getOptionValue={option => option.value}
                                                                valueKey="value"
                                                                {...(formControlProperties?.isMulti ? { components: { Option, ValueContainer } } : {})}
                                                                hideSelectedOptions={false}
                                                                removeSelected={false}
                                                                closeMenuOnSelect={false}
                                                                backspaceRemovesValue={false}
                                                                isSearchable={true}
                                                                allowSelectAll={(formControlProperties?.isMulti || false) ? (formControlProperties?.optionsData || [])?.length > 2 : false}
                                                                onBlur={() => { }}
                                                                noOptionsMessage={() => 'No Data Found'}
                                                                className="select-box"
                                                            />
                                                        </FormGroup>
                                                    </Col>}
                                                {formControlProperties && formControlType === EElementInputTypes.RADIOBUTTON &&
                                                    <Col sm='6' className="hover-action">
                                                        <FormGroup>
                                                            <Label htmlFor={formControlLabel}>{formControlLabel}</Label>
                                                            <span className='float-right d-flex'>
                                                                <i className='icon-edit' onClick={() => { handleFormControlEditDelete(x, EOprationalActions.EDIT) }}></i>
                                                                <i className='icon-delete' onClick={() => { handleFormControlEditDelete(x, EOprationalActions.DELETE) }}></i>
                                                                {/* <i className='icon-Move'></i> */}
                                                            </span>
                                                            {formControlProperties?.optionsData?.map((radioItem, radioItemIndex) => (
                                                                <React.Fragment key={radioItemIndex}>
                                                                    <div className="custom-control custom-radio">
                                                                        <Field type="radio" id={radioItem?.label} name={formControlLabel} value={radioItem?.value} className="custom-control-input" />
                                                                        <label className="custom-control-label" htmlFor={radioItem?.label}>{radioItem?.label}</label>
                                                                    </div>
                                                                </React.Fragment>
                                                            ))}
                                                        </FormGroup>
                                                    </Col>}
                                                {formControlProperties && formControlType === EElementInputTypes.CHECKBOX &&
                                                    <Col sm='6' className="hover-action">
                                                        <FormGroup >
                                                            <Label htmlFor={formControlLabel}>{formControlLabel}</Label>
                                                            <span className='float-right d-flex'>
                                                                <i className='icon-edit' onClick={() => { handleFormControlEditDelete(x, EOprationalActions.EDIT) }}></i>
                                                                <i className='icon-delete' onClick={() => { handleFormControlEditDelete(x, EOprationalActions.DELETE) }}></i>
                                                                {/* <i className='icon-Move'></i> */}
                                                            </span>
                                                            {formControlProperties?.optionsData?.map((checkboxItem, checkboxItemIndex) => (
                                                                <React.Fragment key={checkboxItemIndex}>
                                                                    <div className="custom-control custom-checkbox">
                                                                        <label className="custom-control-label">{checkboxItem?.label}</label>
                                                                        <Field type="checkbox" name={formControlLabel} value={checkboxItem?.value} className="custom-control-input" />
                                                                    </div>
                                                                </React.Fragment>
                                                            ))}
                                                        </FormGroup>
                                                    </Col>}
                                                {formControlProperties && formControlType === EElementInputTypes.DATE &&
                                                    <Col sm='6' className="hover-action">
                                                        <FormGroup>
                                                            <Label htmlFor={formControlLabel}>{formControlLabel}</Label>
                                                            <span className='float-right d-flex'>
                                                                <i className='icon-edit' onClick={() => { handleFormControlEditDelete(x, EOprationalActions.EDIT) }}></i>
                                                                <i className='icon-delete' onClick={() => { handleFormControlEditDelete(x, EOprationalActions.DELETE) }}></i>
                                                                {/* <i className='icon-Move'></i> */}
                                                            </span>
                                                            <DatePicker
                                                                name={formControlLabel}
                                                                className="w100 datepickerIcon form-control"
                                                                placeholderText={formControlLabel}
                                                                popperPlacement="bottom"
                                                                popperModifiers={{
                                                                    flip: {
                                                                        behavior: ["bottom"]
                                                                    },
                                                                    preventOverflow: {
                                                                        enabled: false
                                                                    }
                                                                }}
                                                                selected={x[formControlLabel] ? new Date(x[formControlLabel]) : ''}
                                                                onChange={(e) => { }}
                                                                dateFormat={formControlProperties?.dateFormat || ''}
                                                                minDate={formControlProperties?.fromDate && new Date(formControlProperties?.fromDate)}
                                                                maxDate={formControlProperties?.toDate && new Date(formControlProperties?.toDate)}
                                                                onBlur={() => { }}
                                                                autoComplete="off"
                                                            />
                                                        </FormGroup>
                                                    </Col>}
                                                {!formControlProperties && <Col sm="6">
                                                    <div className='form-adding-type'>
                                                        <i className={x.formControlIcon}></i>
                                                        <span>{x.formControlType}</span>
                                                    </div>
                                                </Col>}
                                            </React.Fragment>
                                        })
                                    }
                                </Row>
                            </div>
                            {digitalFormCreationData && digitalFormRenderData?.length === 0 && <div className='do-message'>
                                <i className='icon-element'></i>
                                {values.error ? <span className='text-danger'>{values.error} </span> : <span>Select an element to add</span>}
                                <small>Nothing is Selected</small>
                            </div>}
                        </div>
                        {digitalFormConfigurations?.isStepperForm && <div className='form-action text-right'>
                            {digitalStepperForms?.length > 1 && <button className='button button-cancel' type='button' onClick={() => handleDeleteDigitalStepperForm(setFieldValue)}>Delete</button>}
                            {digitalStepperForms?.length > 1 && <button className='button button-reset' type='button' onClick={() => handleDigitalFormStepper('previous', { setFieldValue, setFieldTouched, values, setValues })}>Previous</button>}
                            <button className='button button-preview' type='button' onClick={() => handleDigitalFormStepper('next', { setFieldValue, setFieldTouched, values, setValues })}>Next</button>
                        </div>}
                        <div className='form-action'>
                            <button className='button button-cancel' type='button' onClick={handleFormCancel}>Cancel</button>
                            <button className='button button-reset' type='button' onClick={() => handleResetForm(resetForm)}>Reset</button>
                            <button
                                className='button button-preview'
                                type='button'
                                onClick={() => {
                                    if (digitalFormConfigurations?.isStepperForm)
                                        handleDigitalFormStepper('priview', { setFieldValue, setFieldTouched, values, setValues })

                                    else handlePriviewModel()
                                }} disabled={!enableorDisableButtons}>Preview</button>
                            <button className='button button-save' type='submit' disabled={!enableorDisableButtons}>Save</button>
                        </div>
                        <DigitalFormConfigurationModel />
                    </Form>
                }
            }
        </Formik>
    )
}

export default React.memo(DigitalFormAction)