import React, { useRef } from 'react';
import { Row, Col, Label, FormGroup } from 'reactstrap';
import { Field, ErrorMessage, Formik, Form } from 'formik';
import { MySelect, ValueContainer, Option } from '../../helpers/helpersIndex';
import { useTranslation } from 'react-i18next';
import * as _ from 'lodash';
import * as Yup from 'yup';
import { IFormBuilderData, EElementInputTypes } from './digitalFormModel';
import DatePicker from "react-datepicker";
import "react-datepicker/dist/react-datepicker.css";
import { setResetDigitalFormPriviewModel } from '../../store/actions';
import { useDispatch } from 'react-redux';

export interface IProps {
    sampleData: IFormBuilderData[];
    noOfColumn?: number;
    columnSize?: number;
    submitCallBack?: any;
    cancelCallBack?: any;
}
const DigitalFormBuilderRender: React.FC<any> = (props) => {
    console.log('_DynamicFormBuilderRender_props=>', props);
    const dispatch = useDispatch();
    const { t } = useTranslation("translations");
    const buttonsRef = useRef(null) as any;

    const { patchData, digitalFormData, digitalFormValuesValidationSchema,
        stepperIndex, parentSetFieldValue, parentSubmitForm, parentValues,
        cancelCallBack, submitCallBack } = props;
    const { digitalFormConfigurations, digitalFormControls,
        digitalFormTitle, digitalFormId, digitalStepperForms } = digitalFormData;
    const formControlsRenderList = digitalFormConfigurations?.isStepperForm ? [...digitalStepperForms?.[stepperIndex]?.stepperFormControls] : digitalFormControls;
    const columnSize = Math.floor(12 / (digitalFormConfigurations?.noOfColumns || 2));

    const verifyConditionWithParent = (values, parentDependencyValue, childValues) => {
        if (parentDependencyValue) {
            let dataType = typeof parentDependencyValue?.['formControlProperties']?.['dependencyCreation'];
            let value = parentDependencyValue?.['formControlProperties']?.['dependencyCreation'];
            let controlType = parentDependencyValue?.['formControlType'];
            let isMulti = parentDependencyValue?.['formControlProperties']?.['isMulti'];

            if (dataType === 'string' && value === 'isDefined') return values[parentDependencyValue?.formControlId]?.value;
            else if ((controlType === EElementInputTypes.SINGLELINE) && value === values[parentDependencyValue?.formControlId]?.value) return true;
            else if (controlType === EElementInputTypes.RADIOBUTTON && value?.value === values[parentDependencyValue?.formControlId]?.value) return true;
            else if (controlType === EElementInputTypes.CHECKBOX && (values[parentDependencyValue?.formControlId]?.value || [])?.find(x => x === value?.value)) return true;
            else if (controlType === EElementInputTypes.DROPDOWN && isMulti && (values[parentDependencyValue?.formControlId]?.value || [])?.find(x => x?.value === value?.value)) return true;
            else if (controlType === EElementInputTypes.DROPDOWN && value?.value === values[parentDependencyValue?.formControlId]?.value?.value) return true;
            return false
        }
        return true
    }

    const handleDigitalFormClose = () => {
        if (cancelCallBack)
            cancelCallBack()
        else
            dispatch(setResetDigitalFormPriviewModel(false))
    }
    return (
        <div className={`h-100 ${digitalFormConfigurations?.selectedTheme || ""}`}>
            <Formik
                enableReinitialize
                initialValues={{ ...digitalFormValuesValidationSchema?.initialValues, stepperDirection: 'next' }}
                validationSchema={Yup.object().shape({
                    ...digitalFormValuesValidationSchema?.validationSchema
                })}
                onSubmit={(values) => {
                    console.log('values===>', values)
                    const { stepperDirection, ...rest } = values
                    if (digitalFormConfigurations?.isStepperForm) {
                        if (values.stepperDirection === 'next') {
                            parentSetFieldValue?.('stepperIndex', (stepperIndex + 1));
                            parentSetFieldValue?.(`${digitalStepperForms?.[stepperIndex]?.stepperFormId}.value`, rest);
                        }
                        else if (values.stepperDirection === 'previous') {
                            parentSetFieldValue?.('stepperIndex', (stepperIndex - (stepperIndex > 0 ? 1 : 0)));
                            parentSetFieldValue?.(`${digitalStepperForms?.[stepperIndex]?.stepperFormId}.value`, rest);
                        } else if (values.stepperDirection === 'save') {
                            parentSetFieldValue?.(`${digitalStepperForms?.[stepperIndex]?.stepperFormId}.value`, rest);
                            parentSubmitForm();
                        }
                    } else {
                        if (submitCallBack)
                            submitCallBack(rest);
                        else
                            alert(JSON.stringify(rest, undefined, 5));
                    }
                }}
            >
                {({ values, errors, touched, setFieldValue, setFieldTouched, resetForm, handleSubmit, isValid, dirty }) => (
                    <div className="preview-form">
                        {digitalFormTitle && <h6>{digitalFormTitle}</h6>}
                        {digitalFormConfigurations?.isStepperForm && <h6>{digitalStepperForms?.[stepperIndex]?.stepperFormName}</h6>}
                        <Form className='mt-4'>
                            <Row>
                                {formControlsRenderList?.map((control, idx) => {
                                    console.log('formcontrolData==>', control)
                                    // let parentDependency = formControlsRenderList?.find(x => x.formControlId === control?.formControlProperties?.dependencyMapping?.formControlId);
                                    let parentDependency;
                                    let stepper;
                                    let dependencyData;
                                    if (!digitalFormConfigurations?.isStepperForm) {
                                        parentDependency = formControlsRenderList?.find(x => x.formControlId === control?.formControlProperties?.dependencyMapping?.formControlId);
                                        dependencyData = values
                                    } else {
                                        stepper = digitalStepperForms?.find(x => x.stepperFormId === control?.formControlProperties?.dependencyMapping?.stepperFormId);
                                        if (stepper && stepper?.stepperFormId === digitalStepperForms[stepperIndex]?.stepperFormId) {
                                            parentDependency = formControlsRenderList?.find(x => x.formControlId === control?.formControlProperties?.dependencyMapping?.formControlId);
                                            dependencyData = values
                                        } else if (stepper) {
                                            parentDependency = stepper?.stepperFormControls?.find(x => x.formControlId === control?.formControlProperties?.dependencyMapping?.formControlId);
                                            dependencyData = parentValues?.[stepper?.stepperFormId]?.value
                                        }
                                    }
                                    return <React.Fragment key={'row' + 'col' + idx}>
                                        {verifyConditionWithParent(dependencyData, parentDependency, control) &&
                                            <Col sm={columnSize}>
                                                {(control?.formControlType === EElementInputTypes.SINGLELINE) &&
                                                    <FormGroup>
                                                        <Label htmlFor={control?.formControlId}>{control?.formControlLabel}</Label>
                                                        <Field id={control?.formControlId} name={`${control.formControlId}.value`} placeholder={control?.formControlLabel} className='form-control' />
                                                        <ErrorMessage name={`${control.formControlId}.value`} component="div" className="text-danger" />
                                                    </FormGroup>
                                                }
                                                {(control?.formControlType === EElementInputTypes.TEXTAREA) &&
                                                    <FormGroup>
                                                        <Label htmlFor={control?.formControlId}>{control?.formControlLabel}</Label>
                                                        <Field id={control?.formControlId} as='textarea' name={`${control.formControlId}.value`} placeholder={control?.formControlLabel} className='form-control' />
                                                        <ErrorMessage name={`${control.formControlId}.value`} component="div" className="text-danger" />
                                                    </FormGroup>
                                                }
                                                {(control?.formControlType === EElementInputTypes.CHECKBOX) &&
                                                    <>
                                                        <Label className="d-block">{control?.formControlLabel}</Label>
                                                        <FormGroup className="custom-checkbox mb-0">
                                                            {control?.formControlProperties?.optionsData?.map((checkboxItem, checkboxItemIndex) => (
                                                                <React.Fragment key={'row' + 'col' + idx + 'checkbox' + checkboxItemIndex}>
                                                                    <div>
                                                                        <Field type="checkbox" name={`${control.formControlId}.value`} value={checkboxItem?.value} />
                                                                        <label htmlFor={checkboxItem?.label}>{checkboxItem?.label}</label>
                                                                    </div>
                                                                </React.Fragment>
                                                            ))}
                                                            {(errors[control.formControlId]?.['value'] && touched[control.formControlId]?.['value']) && <div className="text-danger">{errors[control.formControlId]?.['value']}</div>}
                                                        </FormGroup>
                                                    </>}
                                                {(control?.formControlType === EElementInputTypes.RADIOBUTTON) &&
                                                    <>
                                                        <Label className="d-block">{control?.formControlLabel}</Label>
                                                        <FormGroup>
                                                            {control?.formControlProperties?.optionsData?.map((radioItem, radioItemIndex) => (
                                                                <React.Fragment key={'row' + 'col' + idx + 'radio' + radioItemIndex}>
                                                                    <div>
                                                                        <Field type="radio" id={radioItem?.label} name={`${control.formControlId}.value`} value={radioItem?.value} />
                                                                        <label htmlFor={radioItem?.label}>{radioItem?.label}</label>
                                                                    </div>

                                                                </React.Fragment>
                                                            ))}
                                                            {(errors[control.formControlId]?.['value'] && touched[control.formControlId]?.['value']) && <div className="text-danger">{errors[control.formControlId]?.['value']}</div>}
                                                        </FormGroup>
                                                    </>
                                                }
                                                {(control?.formControlType === EElementInputTypes.DROPDOWN) &&
                                                    <FormGroup className="ml-2">
                                                        <div className="d-block">{control?.formControlLabel}</div>
                                                        <MySelect
                                                            allOption={{
                                                                label: "Select All",
                                                                value: 0
                                                            }}
                                                            isMulti={control?.formControlProperties?.isMulti || false}
                                                            name={control?.formControlLabel}
                                                            placeholder={control?.formControlLabel}
                                                            options={control?.formControlProperties?.optionsData || []}
                                                            value={values[control?.formControlId]?.value}
                                                            onChange={(e) => setFieldValue(`${control.formControlId}.value`, e ? e : '')}
                                                            getOptionLabel={option => option.label}
                                                            getOptionValue={option => option.value}
                                                            valueKey="value"
                                                            {...(control?.formControlProperties?.isMulti ? { components: { Option, ValueContainer } } : {})}
                                                            {...(!(control?.formControlProperties?.isMulti || control?.formControlProperties?.isRequired) ? { isClearable: true } : {})}
                                                            hideSelectedOptions={false}
                                                            removeSelected={false}
                                                            closeMenuOnSelect={false}
                                                            backspaceRemovesValue={false}
                                                            isSearchable={true}
                                                            allowSelectAll={(control?.formControlProperties?.isMulti || false) ? (control?.formControlProperties?.optionsData || [])?.length > 2 : false}
                                                            onBlur={() => setFieldTouched(`${control.formControlId}.value`, true)}
                                                            noOptionsMessage={() => 'No Data Found'}
                                                            className="select-box"
                                                        />
                                                        {(errors[control.formControlId]?.['value'] && touched[control.formControlId]?.['value']) && (
                                                            <div className="text-danger">{errors[control.formControlId]?.['value']}</div>
                                                        )}
                                                    </FormGroup>}

                                                {(control?.formControlType === EElementInputTypes.DATE) &&
                                                    <FormGroup className="ml-2">
                                                        <Label className="d-block">{control?.formControlLabel}</Label>
                                                        <DatePicker
                                                            name={control?.formControlLabel}
                                                            className="w100 datepickerIcon form-control"
                                                            placeholderText={control?.formControlLabel}
                                                            popperPlacement="bottom"
                                                            popperModifiers={{
                                                                flip: {
                                                                    behavior: ["bottom"]
                                                                },
                                                                preventOverflow: {
                                                                    enabled: false
                                                                }
                                                            }}
                                                            selected={values[control.formControlId]?.value ? new Date(values[control.formControlId]?.value) : ''}
                                                            onChange={(e) => setFieldValue(`${control.formControlId}.value`, e || '')}
                                                            dateFormat={control?.formControlProperties?.dateFormat || ''}
                                                            minDate={control?.formControlProperties?.fromDate && new Date(control?.formControlProperties?.fromDate)}
                                                            maxDate={control?.formControlProperties?.toDate && new Date(control?.formControlProperties?.toDate)}
                                                            onBlur={() => setFieldTouched(`${control.formControlId}.value`, true)}
                                                            autoComplete="off"
                                                        />
                                                        {errors[control.formControlId]?.['value'] && touched[control.formControlId]?.['value'] && (
                                                            <div className="text-danger">{errors[control.formControlId]?.['value']}</div>
                                                        )}
                                                    </FormGroup>}
                                            </Col>}
                                    </React.Fragment>
                                })}
                            </Row>
                            {!digitalFormConfigurations?.isStepperForm && <Row>
                                <div className="form-buttons">
                                    <button className='button button-reset' type='button' onClick={handleDigitalFormClose}>Close</button>
                                    <button className='button button-save' type='submit'>Save</button>
                                </div>
                            </Row>}
                            {digitalFormConfigurations?.isStepperForm && <Row>
                                <div className="form-buttons" ref={buttonsRef}>
                                    <button className='button button-reset' type='button' onClick={handleDigitalFormClose}>Close</button>
                                    {stepperIndex > 0 && <button className='button button-reset' name='previous'
                                        // type='button'
                                        // onClick={() => handleFormSubmission('previous', { values, handleSubmit, isValid, dirty })}
                                        type='submit'
                                        onClick={() => setFieldValue('stepperDirection', 'previous')}
                                    >Previous</button>}
                                    {!(stepperIndex === (digitalStepperForms?.length - 1)) && <button name='next' className='button button-priview'
                                        // type='button'
                                        // onClick={() => handleFormSubmission('next', { values, handleSubmit, isValid, dirty })}
                                        type='submit'
                                        onClick={() => setFieldValue('stepperDirection', 'next')}
                                    >Next</button>}
                                    {(stepperIndex === (digitalStepperForms?.length - 1)) && <button name='save' className='button button-save'
                                        // type='button'
                                        // onClick={() => handleFormSubmission('save', { values, handleSubmit, isValid, dirty })}
                                        type='submit'
                                        onClick={() => setFieldValue('stepperDirection', 'save')}
                                    >Save</button>}
                                </div>
                            </Row>}
                        </Form>
                    </div>
                )}
            </Formik>
        </div>
    )
};
export default React.memo(DigitalFormBuilderRender);