import React from 'react';
import DigitalFormBuilderRender from './digitalFormBuilderRender';
import { EElementInputTypes } from './digitalFormModel';
import { customContentValidation, defultContentValidate, defultContentObjectValidate } from '../../helpers/helpersIndex';
import * as Yup from 'yup';
import { useTranslation } from 'react-i18next';



const digitalFormCreationData = {
    digitalFormTitle: "",
    digitalFormId: '',
    digitalFormConfigurations: {
        noOfColumns: 2,
        isCallbackSave: false,
        isStepperForm: false,
        selectedTheme: 'theme-1',
    },
    digitalFormControls: [],
    digitalStepperForms: []
}

const DigitalFormBuilderRenderContainer: React.FC<any> = (props) => {
    const { t } = useTranslation('translations');
    const { digitalFormCreationData, stepperIndex, parentSetFieldValue, parentValues, parentSubmitForm, patchData, cancelCallBack, submitCallBack } = props

    const { digitalFormTitle, digitalFormId, digitalFormConfigurations, digitalStepperForms, digitalFormControls } = digitalFormCreationData || {}

    const formControlsRenderList = digitalFormConfigurations?.isStepperForm ? [...digitalStepperForms?.[stepperIndex]?.stepperFormControls] : digitalFormControls;
    let existingDataPatch = digitalFormConfigurations?.isStepperForm ? parentValues?.[digitalStepperForms?.[stepperIndex]?.stepperFormId]?.value : patchData
    console.log('existingDataPatch===>', existingDataPatch);

    const verifyValidationBasedonCondition = (parentFormikValue, parentConditionValue, childValues) => {
        let { formControlProperties, formControlType } = parentConditionValue || {};
        let { isMulti, dependencyCreation } = formControlProperties || {};
        console.log('validationData2==>', parentFormikValue, parentConditionValue)
        if (parentFormikValue) {
            if (typeof dependencyCreation === 'string' && dependencyCreation === 'isDefined') return true;
            else if (formControlType === EElementInputTypes.SINGLELINE && dependencyCreation === parentFormikValue) return true;
            else if (formControlType === EElementInputTypes.RADIOBUTTON && dependencyCreation?.value === parentFormikValue) return true;
            else if (formControlType === EElementInputTypes.CHECKBOX && (parentFormikValue || [])?.find(x => x === dependencyCreation?.value)) return true;
            else if (formControlType === EElementInputTypes.DROPDOWN && isMulti && (parentFormikValue || [])?.find(y => y.value === dependencyCreation?.value)) return true;
            else if (formControlType === EElementInputTypes.DROPDOWN && parentFormikValue?.value === dependencyCreation?.value) return true;
        }
        return false
    }
    const digitalformInitialValuesAndValidationSchema = formControlsRenderList?.reduce((total, item) => {

        const { formControlType, formControlId, formControlProperties, formControlLabel } = item;
        const { tag, optionsData, defaultValue, isMulti, requiredMessage, isRequired } = formControlProperties;
        let { initialValues, validationSchema } = total

        // Digital Form initial Values
        if (formControlType === EElementInputTypes.DATE) {
            initialValues = {
                ...initialValues,
                [formControlId]: {
                    label: formControlLabel,
                    value: existingDataPatch ? existingDataPatch?.[formControlId]?.value : '',
                    tag
                }
            };
        }
        else if (formControlType === EElementInputTypes.SINGLELINE || formControlType === EElementInputTypes.TEXTAREA) {
            initialValues = {
                ...initialValues,
                [formControlId]: {
                    tag,
                    label: formControlLabel,
                    value: existingDataPatch ? existingDataPatch?.[formControlId]?.value : (defaultValue || ""),
                }
            }
        }
        else if (formControlType === EElementInputTypes.DROPDOWN || formControlType === EElementInputTypes.RADIOBUTTON || formControlType === EElementInputTypes.CHECKBOX) {
            if (optionsData?.length) {
                const defaultValues = optionsData?.filter(x => x.isdefault);

                let optionsPatchData
                if (formControlType === EElementInputTypes.RADIOBUTTON) {
                    optionsPatchData = optionsData?.filter(x => x.value === existingDataPatch?.[formControlId]?.value);
                } else if (formControlType === EElementInputTypes.CHECKBOX) {
                    console.log('existingDataPatch1===>', existingDataPatch?.[formControlId]);
                    optionsPatchData = optionsData?.filter(x => existingDataPatch?.[formControlId]?.value?.some(y => y === x.value))
                } else {
                    if (isMulti) {
                        optionsPatchData = optionsData?.filter(x => existingDataPatch?.[formControlId]?.value?.some(y => y.value === x.value))
                    }
                    else {
                        optionsPatchData = optionsData?.filter(x => existingDataPatch?.[formControlId]?.value?.value === x.value);
                        console.log('existingDataPatch2===>', existingDataPatch?.[formControlId], optionsPatchData);
                    }
                }

                if (formControlType === EElementInputTypes.DROPDOWN) {
                    initialValues = {
                        ...initialValues,
                        [formControlId]: {
                            tag,
                            label: formControlLabel,
                            value: existingDataPatch ? (optionsPatchData?.length > 1 ?
                                ((isMulti && optionsPatchData?.length > 2) ? [{ label: 'Select All', value: 0 }, ...optionsPatchData] : optionsPatchData) :
                                optionsPatchData?.length === 1 ? (isMulti ? [optionsPatchData?.[0]] : optionsPatchData?.[0]) : '') :
                                (defaultValues?.length > 1 ?
                                    ((isMulti && defaultValues?.length > 2) ? [{ label: 'Select All', value: 0 }, ...defaultValues] : defaultValues) :
                                    defaultValues?.length === 1 ? (isMulti ? [defaultValues?.[0]] : defaultValues?.[0]) : '')
                        }
                    }
                }
                else
                    initialValues = {
                        ...initialValues,
                        [formControlId]: {
                            tag,
                            label: formControlLabel,
                            value: existingDataPatch ? (formControlType === EElementInputTypes.RADIOBUTTON ? (optionsPatchData?.[0]?.value || '') :
                                formControlType === EElementInputTypes.CHECKBOX ? optionsPatchData?.map(x => x.value) : '') :
                                (formControlType === EElementInputTypes.RADIOBUTTON ? (defaultValues?.[0]?.value || '') :
                                    formControlType === EElementInputTypes.CHECKBOX ? defaultValues?.map(x => x.value) : '')
                        }
                    }
            }
            else
                initialValues = { ...initialValues, [formControlId]: '' };
        }


        // Digital Form Validation Schema
        let parentDependency;
        let stepper
        if (!digitalFormConfigurations?.isStepperForm) {
            parentDependency = formControlsRenderList?.find(x => x.formControlId === item?.formControlProperties?.dependencyMapping?.formControlId);
        } else {
            stepper = digitalStepperForms?.find(x => x.stepperFormId === item?.formControlProperties?.dependencyMapping?.stepperFormId);
            if (stepper && stepper?.stepperFormId === digitalStepperForms[stepperIndex]?.stepperFormId) {
                parentDependency = formControlsRenderList?.find(x => x.formControlId === item?.formControlProperties?.dependencyMapping?.formControlId);
            } else if (stepper) {
                parentDependency = stepper?.stepperFormControls?.find(x => x.formControlId === item?.formControlProperties?.dependencyMapping?.formControlId);
            }
        }
        let { formControlId: dependencyFormControlId } = parentDependency || {};

        console.log('ParentDependency===>', parentDependency)

        // dependency Field Schema
        if (parentDependency) {
            if (formControlType === EElementInputTypes.SINGLELINE) {
                validationSchema = {
                    ...validationSchema,
                    [formControlId]: Yup.object().when(`${dependencyFormControlId}.value`, {
                        is: (x) => {
                            console.log('validationData==>', dependencyFormControlId, x, verifyValidationBasedonCondition(x, parentDependency, item));
                            x = x ? x : parentValues?.[stepper?.stepperFormId]?.value?.[dependencyFormControlId]?.value
                            return verifyValidationBasedonCondition(x, parentDependency, item)
                        },
                        then: Yup.object().shape({
                            value: customContentValidation(t, isRequired ? requiredMessage : '', item['formControlProperties']?.['pattern'] ? { patternType: 'customPattern', message: 'customWords', spacialChar: item['formControlProperties']?.['pattern'] } : '', item['formControlProperties']?.['max'], item['formControlProperties']?.['min'])
                        }),
                        otherwise: Yup.object().shape({
                            value: defultContentValidate('')
                        })
                    })
                }
            }
            else if (formControlType === EElementInputTypes.DROPDOWN) {
                if (isMulti) {
                    validationSchema = {
                        ...validationSchema,
                        [formControlId]: Yup.object().when(`${dependencyFormControlId}.value`, {
                            is: (x) => {
                                console.log('validationData==>', dependencyFormControlId, x, verifyValidationBasedonCondition(x, parentDependency, item))
                                return verifyValidationBasedonCondition(x, parentDependency, item)
                            },
                            then: Yup.object().shape({
                                value: Yup.array().required(requiredMessage)
                            }),
                            otherwise: Yup.object().shape({
                                value: Yup.array().notRequired()
                            })
                        })
                    }
                } else {
                    validationSchema = {
                        ...validationSchema,
                        [formControlId]: Yup.object().when(`${dependencyFormControlId}.value`, {
                            is: (x) => {
                                console.log('validationData==>', dependencyFormControlId, x, verifyValidationBasedonCondition(x, parentDependency, item))
                                return verifyValidationBasedonCondition(x, parentDependency, item)
                            },
                            then: Yup.object().shape({
                                value: defultContentObjectValidate(requiredMessage)
                            }),
                            otherwise: Yup.object().shape({
                                value: defultContentObjectValidate('')
                            })
                        })
                    }
                }
            }
            else if (formControlType === EElementInputTypes.RADIOBUTTON) {
                validationSchema = {
                    ...validationSchema,
                    [formControlId]: Yup.object().when(`${dependencyFormControlId}.value`, {
                        is: (x) => {
                            console.log('validationData==>', dependencyFormControlId, x, verifyValidationBasedonCondition(x, parentDependency, item))
                            return verifyValidationBasedonCondition(x, parentDependency, item)
                        },
                        then: Yup.object().shape({
                            value: defultContentValidate(requiredMessage)
                        }),
                        otherwise: Yup.object().shape({
                            value: defultContentValidate('')
                        })
                    })
                }
            }
            else if (formControlType === EElementInputTypes.CHECKBOX) {
                validationSchema = {
                    ...validationSchema,
                    [formControlId]: Yup.object().when(`${dependencyFormControlId}.value`, {
                        is: (x) => {
                            console.log('validationData==>', dependencyFormControlId, x, verifyValidationBasedonCondition(x, parentDependency, item))
                            return verifyValidationBasedonCondition(x, parentDependency, item)
                        },
                        then: Yup.object().shape({
                            value: Yup.array(Yup.string()).required(requiredMessage)
                        }),
                        otherwise: Yup.object().shape({
                            value: Yup.array().notRequired()
                        })
                    })
                }
            }
        } // Normal Fields Schema
        else {
            if (formControlType === EElementInputTypes.DATE) {
                validationSchema = {
                    ...validationSchema,
                    [formControlId]: Yup.object().shape({
                        value: isRequired ? defultContentValidate(requiredMessage) : ''
                    })
                };
            }
            else if (formControlType === EElementInputTypes.SINGLELINE || formControlType === EElementInputTypes.TEXTAREA) {
                validationSchema = {
                    ...validationSchema,
                    [formControlId]: Yup.object().shape({
                        value: customContentValidation(t, isRequired ? requiredMessage : '', item['formControlProperties']?.['pattern'] ? { patternType: 'customPattern', message: 'customWords', spacialChar: item['formControlProperties']?.['pattern'] } : '', item['formControlProperties']?.['max'], item['formControlProperties']?.['min'])
                    })
                }
            }
            else if (formControlType === EElementInputTypes.DROPDOWN) {
                if (isMulti) {
                    validationSchema = {
                        ...validationSchema,
                        [formControlId]: Yup.object().shape({
                            value: isRequired ? Yup.array().required(requiredMessage) : ''
                        })
                    }
                } else {
                    validationSchema = {
                        ...validationSchema,
                        [formControlId]: Yup.object().shape({
                            value: isRequired ? defultContentObjectValidate(requiredMessage) : ''
                        })
                    }
                }
            }
            else if (formControlType === EElementInputTypes.RADIOBUTTON) {
                validationSchema = {
                    ...validationSchema,
                    [formControlId]: Yup.object().shape({
                        value: isRequired ? defultContentValidate(requiredMessage) : ''
                    })
                }
            }
            else if (formControlType === EElementInputTypes.CHECKBOX) {
                validationSchema = {
                    ...validationSchema,
                    [formControlId]: Yup.object().shape({
                        value: isRequired ? Yup.array(Yup.string()).required(requiredMessage) : ''
                    })
                }
            }
        }

        total.initialValues = initialValues
        total.validationSchema = validationSchema
        return total
    }, {
        initialValues: {},
        validationSchema: {}
    });

    console.log("DigitalFormBuilderRender", digitalformInitialValuesAndValidationSchema)
    return (
        <>
            <DigitalFormBuilderRender
                digitalFormData={digitalFormCreationData}
                digitalFormValuesValidationSchema={digitalformInitialValuesAndValidationSchema}
                stepperIndex={stepperIndex}
                parentSetFieldValue={parentSetFieldValue}
                parentValues={parentValues}
                parentSubmitForm={parentSubmitForm}
                cancelCallBack={cancelCallBack}
                submitCallBack={submitCallBack}
                {...(stepperIndex !== undefined ? { key: stepperIndex } : {})}
            />
        </>
    )
}



export default React.memo(DigitalFormBuilderRenderContainer);
