import React from 'react';
import { useDispatch, useSelector } from 'react-redux';
import { Modal, ModalHeader, ModalBody, ModalFooter } from 'reactstrap';
import { Field, Form, Formik, ErrorMessage } from 'formik';
import { FormGroup, Label } from 'reactstrap';
import { v4 as uuidv4 } from 'uuid';
import { IFormBuilderReducer } from './digitalFormModel';
import { MySelect, defultContentValidate } from '../../helpers/helpersIndex';
import { setResetDigitalFormConfigurationsModel, setFormBuilderActionTypeAndActionData } from '../../store/actions';
import { EOprationalActions } from '../../models/utilitiesModel';
import * as Yup from 'yup';


const DigitalFormConfigurationModel: React.FC<any> = () => {
    const dispatch = useDispatch();
    const digitalFormCreationData: any = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.digitalFormCreationData);
    const isModelOpen: boolean = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.isDigitalFormConfigModelOpen);

    const { digitalFormConfigurations, digitalFormControls, digitalStepperForms } = digitalFormCreationData || {};

    const cloumDropDownOptions = [
        { label: '1-column', value: 1 },
        { label: '2-column', value: 2 },
        { label: '3-column', value: 3 },
        { label: '4-column', value: 4 },
    ];

    const themeOptions = [
        { label: 'Theme-1', value: 'theme-1' },
        { label: 'Theme-2', value: 'theme-2' },
        { label: 'Theme-3', value: 'theme-3' },
        { label: 'Theme-4', value: 'theme-4' },
        { label: 'Theme-5', value: 'theme-5' },
    ]

    const handleFormConfigurationModelClose = () => {
        dispatch(setResetDigitalFormConfigurationsModel(false))
    }

    const handleAPIHeadersSave = (values, setFieldValue) => {
        if (values.headerValue && values.headerName) {
            let headersValues = {
                ...values.apiHeaders,
                [values.headerName]: values.headerValue
            }
            setFieldValue('apiHeaders', headersValues);
            setFieldValue('headerName', '');
            setFieldValue('headerValue', '');
            if (values.isHeaderEdit) {
                setFieldValue('isHeaderEdit', false)
            }
        }
    }

    const handleHeaderEdit = (header, values, setFieldValue) => {
        setFieldValue('headerName', header);
        setFieldValue('headerValue', values.apiHeaders[header]);
        setFieldValue('isHeaderEdit', true)
    }

    const handleHeaderDelete = (header, values, setFieldValue) => {
        const { [header]: Hdata, ...rest } = values.apiHeaders;
        setFieldValue('apiHeaders', rest);
    }

    return (
        <Modal isOpen={isModelOpen} className="modal-md">
            <ModalHeader>{'Digital Form Configuration'}</ModalHeader>
            <ModalBody>
                <Formik
                    enableReinitialize
                    initialValues={{
                        noOfColumns: digitalFormConfigurations?.noOfColumns,
                        isDigitalSave: digitalFormConfigurations?.isDigitalSave,
                        isStepperForm: digitalFormConfigurations?.isStepperForm,
                        selectedTheme: digitalFormConfigurations?.selectedTheme,
                        isWebHook: digitalFormConfigurations?.isWebHook,
                        api: digitalFormConfigurations?.api,
                        headerName: '',
                        headerValue: "",
                        isHeaderEdit: false,
                        apiHeaders: digitalFormConfigurations?.apiHeaders || {}
                    }}
                    validationSchema={Yup.object().shape({
                        api: Yup.string().when('isWebHook', {
                            is: (x) => x,
                            then: defultContentValidate('This Field is Required'),
                            otherwise: defultContentValidate('')
                        })
                    })}
                    onSubmit={(values) => {
                        let data = {
                            ...digitalFormCreationData,
                            digitalFormConfigurations: {
                                noOfColumns: values.noOfColumns,
                                isDigitalSave: values.isDigitalSave,
                                isStepperForm: values.isStepperForm,
                                selectedTheme: values.selectedTheme,
                                isWebHook: values.isWebHook,
                                api: values.api,
                                apiHeaders: values.apiHeaders
                            },
                            ...(values.isStepperForm ?
                                {
                                    digitalStepperForms: [
                                        {
                                            stepperFormId: uuidv4(),
                                            stepperFormName: '',
                                            stepperFormControls: [...(digitalFormControls || [])]
                                        }
                                    ],
                                    digitalFormControls: []
                                } :
                                {
                                    digitalFormControls: [
                                        ...digitalFormControls,
                                        ...digitalStepperForms?.flatMap(x => x.stepperFormControls)
                                    ],
                                    digitalStepperForms: []
                                }
                            )
                        }
                        dispatch(setFormBuilderActionTypeAndActionData(EOprationalActions.ADD, data));
                        dispatch(setResetDigitalFormConfigurationsModel(false))
                    }}
                >
                    {
                        ({ values, setFieldValue, setFieldTouched }) => {
                            return <Form >
                                <FormGroup>
                                    <Label htmlFor="theme">Select No.of Columns</Label>
                                    <MySelect
                                        name="theme"
                                        id="theme"
                                        options={cloumDropDownOptions}
                                        value={cloumDropDownOptions?.find(x => x.value === values.noOfColumns) || ''}
                                        onChange={(e) => setFieldValue('noOfColumns', e ? e?.value : '')}
                                        placeholder="Select No.of Columns"
                                        hideSelectedOptions={false}
                                        valueKey='value'
                                        removeSelected={false}
                                        closeMenuOnSelect={true}
                                        backspaceRemovesValue={false}
                                        onBlur={() => setFieldTouched('noOfColumns', true)}
                                        className="select-box"
                                    />
                                    <ErrorMessage name="noOfColumns" component="div" className="text-danger" />
                                </FormGroup>
                                <FormGroup>
                                    <Label htmlFor="theme">Select Theme</Label>
                                    <MySelect
                                        name="theme"
                                        id="theme"
                                        options={themeOptions}
                                        value={themeOptions?.find(x => x.value === values.selectedTheme) || ''}
                                        onChange={(e) => setFieldValue('selectedTheme', e ? e?.value : '')}
                                        placeholder="Select Theme"
                                        hideSelectedOptions={false}
                                        valueKey='value'
                                        removeSelected={false}
                                        closeMenuOnSelect={true}
                                        backspaceRemovesValue={false}
                                        onBlur={() => setFieldTouched('selectedTheme', true)}
                                        className="select-box"
                                    />
                                    <ErrorMessage name="selectedTheme" component="div" className="text-danger" />
                                </FormGroup>
                                <FormGroup>
                                    <Label >is StepperForm</Label> : &nbsp;
                                    <Field type='checkbox' name='isStepperForm' />
                                </FormGroup>

                                <FormGroup>
                                    <Label>is Digital Save</Label> : &nbsp;
                                    <Field type='checkbox' name='isDigitalSave' />
                                </FormGroup>

                                <FormGroup>
                                    <Label >is Web Hook</Label> : &nbsp;
                                    <Field type='checkbox' name='isWebHook' />
                                </FormGroup>
                                {values.isWebHook && <>
                                    <FormGroup>
                                        <Label htmlForm='api'>API</Label>
                                        <Field id="api" type="text" placeholder="Enter API URL" name="api" className="form-control" />
                                        <ErrorMessage name="api" component="div" className="text-danger" />
                                    </FormGroup>
                                    <FormGroup>
                                        <Label >API Headers</Label>
                                        <div className='d-flex'>
                                            <Field id="headerName" type="text" disabled={values.isHeaderEdit} placeholder="Enter Header Name" name="headerName" className="form-control" />
                                            <Field id="headerValue" type="text" placeholder="Enter Header Value" name="headerValue" className="form-control" />
                                            <button className='btn btn-primary btn-sm' type='button' onClick={() => handleAPIHeadersSave(values, setFieldValue)}>Save</button>{' '}
                                        </div>
                                    </FormGroup>
                                    {Object.keys(values.apiHeaders)?.length > 0 && <table className='table'>
                                        <thead>
                                            <th>Header Name</th>
                                            <th>Header Value</th>
                                            <th>Actions</th>
                                        </thead>
                                        <tbody>
                                            {
                                                Object.keys(values.apiHeaders)?.map(x => {
                                                    return <tr>
                                                        <td>{x}</td>
                                                        <td>{values.apiHeaders[x]}</td>
                                                        <td>
                                                            <i className='icon-edit' onClick={() => handleHeaderEdit(x, values, setFieldValue)}></i>
                                                            <i className='icon-delete' onClick={() => handleHeaderDelete(x, values, setFieldValue)}></i>
                                                        </td>
                                                    </tr>
                                                })
                                            }
                                        </tbody>
                                    </table>}
                                </>}

                                <ModalFooter>
                                    <button className='btn btn-primary' type='submit'>Save</button>{' '}
                                    <button className='btn btn-danger' type='button' onClick={handleFormConfigurationModelClose}>Cancel</button>
                                </ModalFooter>
                            </Form>
                        }
                    }
                </Formik>
            </ModalBody>
        </Modal>
    );
};

export default React.memo(DigitalFormConfigurationModel);