import React, { useState } from 'react';
import { useSelector, useDispatch } from 'react-redux';
import DigitalFormCreateOrEditManager from './digitalFormCreateOrEditManager';
import { EAPPModules, EOprationalActions } from '../../models/utilitiesModel';
import DigitalFormView from './digitalFormView';
import RenderForm from './RenderForm';
import { IFormBuilderModel, IFormBuilderReducer } from './digitalFormModel';
import bgimage from '../../images/formIMG.svg';
import { setFormBuilderActionTypeAndActionData } from '../../store/actions';


const DigitalFormManager = () => {
    const dispatch = useDispatch();
    const formOptions: IFormBuilderModel[] = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.formBuilderData || []);
    const actionType: number = useSelector((state: any) => (state.dynamicFormBuilderReducer as IFormBuilderReducer)?.actionType);
    const [renderFormId, setRenderFormId] = useState('');

    const cancelCallBack = () => {
        setRenderFormId('');
    }

    const digitalFormCreationData = {
        digitalFormTitle: "",
        digitalFormId: '',
        digitalFormConfigurations: {
            noOfColumns: 2,
            isCallbackSave: false,
            isStepperForm: false,
            selectedTheme: 'theme-1',
            isWebHook: false
        },
        digitalFormControls: [],
        digitalStepperForms: []
    }

    const addForm = () => dispatch(setFormBuilderActionTypeAndActionData(EOprationalActions.ADD, digitalFormCreationData));
    console.log(`${EAPPModules.DYNAMICFORMBUILDER}_DynamicFormManager==>`, formOptions)

    return (
        <>
            {
                !Boolean(renderFormId) && <>
                    {actionType === EOprationalActions.UNSELECT && <div className='homepage'>
                        <div className='homepage-left'>
                            <div className="p-4">
                                <span>Build Your forms with</span>
                                <h3 className="mt-0">VM Form Builder</h3>
                                <small>Build powerful and secure forms - without code to integrate in your application or website</small>
                            </div>
                            <img src={bgimage} alt='' />
                        </div>
                        <div className='homepage-right'>
                            <div className='homepage-right-header mb-3'>
                                <h6>My Forms</h6>
                                <button className='dashed-button' onClick={addForm}><i className='icon-add'></i>Create Form</button>
                            </div>
                            <div className="d-flex flex-column flexLayout">
                                <div className="flexScroll">
                                    <div className="table-responsive-sm fixTableHead">
                                        <table className="table fb-tbl">
                                            <thead>
                                                <tr>
                                                    <th>Status</th>
                                                    <th>Form Name</th>
                                                    <th>Created Date</th>
                                                    <th>Actions</th>
                                                </tr>
                                            </thead>
                                            <tbody>
                                                {formOptions?.map((form, index) => <React.Fragment key={form.formBuilderTitle}>
                                                    <DigitalFormView {...{ id: form.id, setRenderFormId }} />
                                                </React.Fragment>)}
                                            </tbody>
                                        </table>
                                        {formOptions && formOptions?.length === 0 && <h6 className='text-center'>No Form Records Found</h6>}
                                    </div>
                                </div>
                                <div className="pagination paging-pad">

                                </div>
                            </div>
                            {/* <div className='text-right pt-2'>&copy; 2022 Vectramind All rights reserved</div> */}
                        </div>
                    </div>}
                    {/* <br /> */}
                    {actionType !== EOprationalActions.UNSELECT && <DigitalFormCreateOrEditManager />}
                </>}
            {renderFormId && <RenderForm {...{ id: renderFormId, cancelCallBack }} />}
        </>
    )
}
export default React.memo(DigitalFormManager);