import { getHost } from '../../helpers/helpersIndex';
import { EAPPModules, EOprationalActions, IConfirmModel, IConfirmOptions } from '../../models/utilitiesModel';
import React, { useState } from 'react';
import { useSelector, useDispatch } from 'react-redux';
import { deleteFormFormBuilderRequest, setConfirmationOpen, setFormBuilderActionTypeAndActionData } from '../../store/actions';
import { IFormBuilderModel } from './digitalFormModel';
import Switch from "react-switch";
import cloneDeep from 'lodash/cloneDeep';


const DigitalFormView = ({ id, setRenderFormId }) => {
    const dispatch = useDispatch();
    const [checked, setChecked] = useState(false);

    const form: IFormBuilderModel | undefined = useSelector((state: any) => (state.dynamicFormBuilderReducer?.formBuilderData || [])?.find(x => x.id === id));
    const digitalFormCreationData = {
        digitalFormTitle: form?.formBuilderTitle || "",
        digitalFormId: form?.id || '',
        digitalFormConfigurations: {
            ...cloneDeep(form?.formBuilderConfigurations)
        },
        digitalFormControls: !form?.formBuilderConfigurations?.isStepperForm ? cloneDeep(form?.formBuilderData) : [],
        digitalStepperForms: form?.formBuilderConfigurations?.isStepperForm ? cloneDeep(form.formBuilderData) : [],
    }
    const editForm = (form) => dispatch(setFormBuilderActionTypeAndActionData(EOprationalActions.ADD, digitalFormCreationData));
    const deleteForm = (id) => {
        id && dispatch(deleteFormFormBuilderRequest(id));
    }
    console.log(`${EAPPModules.DYNAMICFORMBUILDER}_DynamicFormView==>`, id)
    const generateUrl = (id) => {
        if (id) {
            let optionsData = [
                {
                    title: 'copy',
                    loading: 0,
                    className: 'btn-info'
                },
                {
                    title: 'cancel',
                    loading: 0,
                    className: 'btn-danger'
                }
            ] as IConfirmOptions[];
            let confirmModel = {
                title: 'Generate URL',
                subtitle: `${getHost()}form?id=${id}`,
                options: optionsData,
                transKey: '1234'
            } as IConfirmModel;
            dispatch(setConfirmationOpen(confirmModel));
        }
    }
    return (
        <>{form &&
            <tr>
                <td>
                    <div className={'fix-plan ' + (checked ? 'switch-transform' : '')}>
                        <Switch onChange={() => setChecked(true)} checkedIcon={false} uncheckedIcon={false} checked={checked} />
                    </div>
                </td>
                <td>{form.formBuilderTitle}</td>
                <td>{form.createdOn ? new Date(form.createdOn)?.toLocaleDateString() : '-'}</td>
                <td>
                    <div className='d-flex'>
                        <i className='icon-edit' onClick={() => editForm(form)}></i>
                        <i className='icon-renderForm' onClick={() => setRenderFormId(form.id)}></i>
                        <i className='icon-link' onClick={() => generateUrl(form.id)}></i>
                        <i className='icon-delete' onClick={() => deleteForm(form.id)}></i>
                    </div>
                </td>
            </tr>}
        </>)
}
export default React.memo(DigitalFormView);